unit AnalogInstrument;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, EmpfangPrototyp, Hauptfenster,
  IntegerTerm, AnalogKonfig;

type
  TAnalogInstrumentForm = class(TEmpfangPrototypForm)
    KonfigBtn: TBitBtn;
    AktivChecker: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure KonfigBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    BerechnungsObj: TIntTerm;
    Varis: VarArray;

    SchattenBild: TBitmap;  // Speichert das Bild im Hintergrund
    SkalenRadius: integer;
    SkalenTeilung: integer;
    SkalenSubTeilung: integer;
    StartWinkel: integer;    // Winkel an dem die Skala beginnt 12Uhr = 0 Grad
    SkalenWinkel: integer;
    Margin: integer;       // Abstand der Skala zum Rand
    MinWert: integer;
    MaxWert: integer;
    ZeigerWert: integer;
    SkalenFaktor: integer;
    Einheit: string;
    NachrichtenID: longword;
    BerechnungsTerm: string;
//    Refreshrate: integer;
    procedure GetKreispunkt(w: real; r: integer; x, y: pinteger);
    procedure ZeichneSkala();
    procedure ZeichneZeiger();
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

implementation

{$R *.dfm}

procedure TAnalogInstrumentForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  inherited;
  TMainForm(owner).CheckerImages.GetBitmap(5, KonfigBtn.Glyph);

  BerechnungsObj := TIntTerm.Create;

  SetLength(Varis, 8);
  for i := 0 to high(Varis) do
  begin
    Varis[i].Name := format('d%u',[i]);
    Varis[i].Wert := 0;
  end;

  SkalenRadius := 125;
  SkalenTeilung := 11;
  SkalenSubTeilung := 5;
  StartWinkel := -135;
  SkalenWinkel := 270;
  Margin := 5;
  MinWert := 0;
  MaxWert := 10000;
  ZeigerWert := 0;
  SkalenFaktor:= 1000;
  Einheit := 'x';
  NachrichtenID := 0;
  BerechnungsTerm := '(d0 << 8) + d1';

  SchattenBild := TBitmap.Create;
  SchattenBild.PixelFormat := pf32bit;
  SchattenBild.Width := SkalenRadius * 2;
  SchattenBild.Height := SkalenRadius * 2;

  self.ClientWidth := (SkalenRadius + Margin) * 2;
  self.ClientHeight := (SkalenRadius + Margin) * 2;

  ZeichneSkala;
end;

procedure TAnalogInstrumentForm.FormDestroy(Sender: TObject);
begin
  inherited;
  SchattenBild.Free;
end;

procedure TAnalogInstrumentForm.FormResize(Sender: TObject);
begin
  inherited;
  KonfigBtn.Left := self.ClientWidth - KonfigBtn.Width;
end;

procedure TAnalogInstrumentForm.GetKreispunkt(w: real; r: integer; x, y: pinteger);
begin
  x^ := round(sin((w + StartWinkel) * pi / 180) * r);
  y^ := round(cos((w + StartWinkel) * pi / 180) * r) * -1;
end;

procedure TAnalogInstrumentForm.ZeichneSkala;
var
  x,y,xm,ym,i: integer;
  w: Single;
  s: string;
begin
  SchattenBild.Canvas.Brush.Color := self.Color;
  SchattenBild.Canvas.FillRect(Rect(0, 0, SchattenBild.Width-1, SchattenBild.Height-1));
  SchattenBild.Canvas.Pen.Color := clblack;
  SchattenBild.Canvas.Pen.Style := psSolid;
  SchattenBild.Canvas.Font.Color := clRed;

  xm := SkalenRadius - 1;
  ym := SkalenRadius - 1;
  i := 0;
  while (i <= ((SkalenTeilung - 1)* SkalenSubTeilung)) do
  begin
    w := i * (SkalenWinkel / ((SkalenTeilung -1) * SkalenSubTeilung));
    GetKreispunkt(w, SkalenRadius - 2, @x, @y);
    SchattenBild.Canvas.MoveTo(xm + x, xm + y);
    if (i mod SkalenSubTeilung) = 0 then
    begin
      GetKreispunkt(w, SkalenRadius - 11, @x, @y);
      SchattenBild.Canvas.Pen.Width := 2;
    end
    else
    begin
      GetKreispunkt(w, SkalenRadius - 7, @x, @y);
      SchattenBild.Canvas.Pen.Width := 1;
    end;
    SchattenBild.Canvas.LineTo(xm + x, xm + y);
    inc(i);
  end;

  i := 0;
  while (i <= (SkalenTeilung - 1)) do
  begin
    s := format('%d',[round((MinWert + i * ((MaxWert - MinWert) / (SkalenTeilung - 1))) / SkalenFaktor)]);
    w := i * (SkalenWinkel / (SkalenTeilung - 1));
    GetKreispunkt(w, SkalenRadius - 15, @x, @y);
    if (x > -45) then
      if (x < 45) then
        x := x - SchattenBild.Canvas.TextWidth(s) div 2
      else
        x := x - SchattenBild.Canvas.TextWidth(s);

    if (y > -45) then
      if (y < 45) then
        y := y - SchattenBild.Canvas.TextHeight(s) div 2
      else
        y := y - SchattenBild.Canvas.TextHeight(s);


    SchattenBild.Canvas.TextOut(xm + x, ym + y, s);
    inc(i);
  end;

  SchattenBild.Canvas.Ellipse(xm -3, ym - 3, xm + 2, ym + 2);

  s := Einheit;
  if SkalenFaktor > 1 then
    s := format('x%d %s',[SkalenFaktor, s]);
  x := SchattenBild.Canvas.TextWidth(s) div 2;
  SchattenBild.Canvas.TextOut(xm - x, ym + 20, s);
end;

procedure TAnalogInstrumentForm.ZeichneZeiger();
var
  x,y,xm,ym,r1,wert: integer;

begin
  xm := SkalenRadius + Margin - 2;
  ym := SkalenRadius + Margin - 2;
  r1 := SkalenRadius - 12;
  wert := ZeigerWert;
  if wert < MinWert then
    wert := MinWert
  else
    if wert > MaxWert then
      wert := MaxWert;

  GetKreispunkt((wert - MinWert) * (SkalenWinkel / (MaxWert - MinWert)), r1, @x, @y);
  self.Canvas.Pen.Width := 2;
  self.Canvas.MoveTo(xm, ym);
  self.Canvas.LineTo(x + xm, y + ym);
end;

procedure TAnalogInstrumentForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
begin
  if (not AktivChecker.Checked) or (CANrec.ID <> NachrichtenID) then
    exit;

  for i := 0 to high(Varis) do
    Varis[i].Wert := CANrec.Data[i];

  try
    ZeigerWert := BerechnungsObj.TermLoesen(BerechnungsTerm, @Varis);
    self.Paint;
  except
    AktivChecker.Checked := false;
    MessageDlg('Im Berechnungs Term ist ein Fehler aufgetretten:'+#13+#10+
          BerechnungsObj.GetFehlerText, mtError, [mbOk], 0);
  end;
end;

procedure TAnalogInstrumentForm.FormPaint(Sender: TObject);
begin
  BitBlt(self.Canvas.Handle, Margin, Margin, SchattenBild.Width-1, SchattenBild.Height-1,
        SchattenBild.Canvas.Handle, 0, 0, SRCCOPY);

  ZeichneZeiger();
end;

procedure TAnalogInstrumentForm.KonfigBtnClick(Sender: TObject);
var
  Form: TAnalogKonfigForm;
begin
  Form := TAnalogKonfigForm.Create(self);

  Form.NameEdit.Text := self.Caption;
  Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
  Form.EinheitEdit.Text := Einheit;
  Form.BerechnungsTermEdit.Text := BerechnungsTerm;
  Form.MinSkalaSpin.Value := MinWert;
  Form.MaxSkalaSpin.Value := MaxWert;
  Form.SkalaUnterteilungSpin.Value := SkalenSubTeilung;
  Form.SkalaTeilungSpin.Value := SkalenTeilung;
  Form.FaktorSpin.Value := SkalenFaktor;

  if Form.ShowModal = mrOK then
  begin
    self.Caption := Form.NameEdit.Text;
    NachrichtenID := Form.CANIDEdit.Tag;
    Einheit := Form.EinheitEdit.Text;
    BerechnungsTerm := Form.BerechnungsTermEdit.Text;
    MinWert := Form.MinSkalaSpin.Value;
    MaxWert := Form.MaxSkalaSpin.Value;
    SkalenSubTeilung := Form.SkalaUnterteilungSpin.Value;
    SkalenTeilung := Form.SkalaTeilungSpin.Value;
    SkalenFaktor := Form.FaktorSpin.Value;

    ZeichneSkala;
    self.Paint;
    FensterMenuItem.Caption :=  self.Caption;
  end;
  Form.Free;
end;

procedure TAnalogInstrumentForm.KonfigLaden(WerteListe: TStrings);
begin
  self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
  NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
  Einheit := ReadListString(WerteListe, 'Einheit', Einheit);
  BerechnungsTerm := ReadListString(WerteListe, 'Term', BerechnungsTerm);
  MinWert := ReadListInteger(WerteListe, 'MinWert', MinWert);
  MaxWert := ReadListInteger(WerteListe, 'MaxWert', MaxWert);
  SkalenTeilung := ReadListInteger(WerteListe, 'SkalenTeilung', SkalenTeilung);
  SkalenSubTeilung := ReadListInteger(WerteListe, 'SkalenSubTeilung', SkalenSubTeilung);
  SkalenFaktor := ReadListInteger(WerteListe, 'SkalenFaktor', SkalenFaktor);

  ZeichneSkala;
  self.Paint;
  FensterMenuItem.Caption :=  self.Caption;
end;

procedure TAnalogInstrumentForm.KonfigSichern(WerteListe: TStrings);
begin
  WerteListe.Append(format('Name=%s', [self.Caption]));
  WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
  WerteListe.Append(format('Einheit=%s', [Einheit]));
  WerteListe.Append(format('Term=%s', [BerechnungsTerm]));
  WerteListe.Append(format('MinWert=%d', [MinWert]));
  WerteListe.Append(format('MaxWert=%d', [MaxWert]));
  WerteListe.Append(format('SkalenTeilung=%u', [SkalenTeilung]));
  WerteListe.Append(format('SkalenSubTeilung=%u', [SkalenSubTeilung]));
  WerteListe.Append(format('SkalenFaktor=%u', [SkalenFaktor]));
end;

end.
