unit Empfang;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, StdCtrls, Buttons, Hauptfenster, ExtCtrls, StrUtils,
  EmpfangPrototyp;

type
  TEmpfangForm = class(TEmpfangPrototypForm)
    NachrichtenGrid: TStringGrid;
    KopfPanel: TPanel;
    AnsichtGroup: TGroupBox;
    IDListRadio: TRadioButton;
    ProtokollRadio: TRadioButton;
    ListeLeerenBtn: TBitBtn;
    SpeichernBtn: TBitBtn;
    RefreshTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure NachrichtenGridDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure FormDestroy(Sender: TObject);
    procedure ListeLeeren(Sender: TObject);
    procedure SpeichernBtnClick(Sender: TObject);
    procedure RefreshTimerTimer(Sender: TObject);
  private
    { Private-Deklarationen }
    NachrichenListe: TList;
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure BerechneColWidth();
    procedure SetColCount();
  end;

implementation

{$R *.dfm}

{ TEmpfangForm }

procedure TEmpfangForm.FormCreate(Sender: TObject);
begin
  inherited;
  TMainForm(owner).ButtonImages.GetBitmap(1, ListeLeerenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(4, SpeichernBtn.Glyph);

  NachrichenListe := TList.Create;
  NachrichenListe.Capacity := 1024;

  SetColCount;
  NachrichtenGrid.ColWidths[0] := 80;
  NachrichtenGrid.ColWidths[1] := 27;
  BerechneColWidth;
  NachrichtenGrid.ColWidths[10] := 30;
  NachrichtenGrid.ColWidths[11] := 80;
//  NachrichtenGrid.Rows[0].CommaText := 'ID,DLC,D0,D1,D2,D3,D4,D5,D6,D7,Rm,Zeitst.';
  NachrichtenGrid.Hint := 'ID - Idenifier'+#13+
                      'DLC - Lnge der CAN-Nachricht'+#13+
                      'Dx - Datenbytes'+#13+
                      'Rm - Remote'+#13+
                      'Zeitst. - Zeitstempel'+#13+
                      'ASCII - Daten als Zeichen des ASCII-Codes';
end;

procedure TEmpfangForm.FormDestroy(Sender: TObject);
begin
  ListeLeeren(self);
  NachrichenListe.Free;
end;

procedure TEmpfangForm.ListeLeeren(Sender: TObject);
begin
  while (NachrichenListe.Count > 0) do
  begin
    dispose(NachrichenListe.Items[0]);
    NachrichenListe.Delete(0);
  end;
  NachrichtenGrid.RowCount := 2;
  NachrichtenGrid.Refresh;
end;

procedure TEmpfangForm.NachrichtenGridDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  Text: string;
  x,y: integer;
  i: integer;
begin
  NachrichtenGrid.Canvas.Brush.Color := NachrichtenGrid.Color;
  NachrichtenGrid.Canvas.Font.Color := clWindowText;

  if gdFocused in State then
  begin
    NachrichtenGrid.Canvas.Brush.Color := clActiveCaption;
    NachrichtenGrid.Canvas.Font.Color := clCaptionText;
  end
  else
    if gdFixed in State then
      NachrichtenGrid.Canvas.Brush.Color := NachrichtenGrid.FixedColor;
  NachrichtenGrid.Canvas.FillRect(Rect);

  if ARow = 0 then
  begin
    case ACol of
      0:  Text := 'ID';
      1:  Text := 'DLC';
      2:  Text := 'D0';
      3:  Text := 'D1';
      4:  Text := 'D2';
      5:  Text := 'D3';
      6:  Text := 'D4';
      7:  Text := 'D5';
      8:  Text := 'D6';
      9:  Text := 'D7';
      10: Text := 'Rm';
      11: Text := 'Zeitst.';
      12: Text := 'ASCII';
    end;
  end
  else
  begin
    if ARow <= NachrichenListe.Count then
    begin
      case ACol of
        0:  Text := TMainForm(Owner).GetIDString(PCANrecord(NachrichenListe[ARow-1]).ID);

        1:  Text := format('%u',[PCANrecord(NachrichenListe[ARow-1]).DLC]);

        2..9:
          begin
            if (ACol-1) > PCANrecord(NachrichenListe[ARow-1]).DLC then
              exit;
            Text := TMainForm(Owner).GetDataString(PCANrecord(NachrichenListe[ARow-1]).Data[ACol-2]);
          end;

        10:
          begin     //  Remote
            x := (NachrichtenGrid.ColWidths[ACol] - TMainForm(Owner).CheckerImages.Width) div 2;
            y := (NachrichtenGrid.DefaultRowHeight - TMainForm(Owner).CheckerImages.Height) div 2;
            TMainForm(self.Owner).CheckerImages.Draw(NachrichtenGrid.Canvas,Rect.Left+x,Rect.Top+y,(PCANrecord(NachrichenListe[ARow-1]).Cmd shr 5) and $01);
            exit;
          end;

        11:
          begin
            if ProtokollRadio.Checked then
              Text := format('%u',[PCANrecord(NachrichenListe[ARow-1]).Timestamp])
            else
              if (PCANrecord(NachrichenListe[ARow-1]).MsgAnzahl > 0) then
                Text := format('%u',[PCANrecord(NachrichenListe[ARow-1]).IntervalSumme div PCANrecord(NachrichenListe[ARow-1]).MsgAnzahl])
              else
                Text := '-';
          end;
        12:
          begin
            NachrichtenGrid.Canvas.Font.Name := 'Terminal';
            Text := '        ';
            for i := PCANrecord(NachrichenListe[ARow-1]).DLC - 1 downto 0 do
            begin
              if PCANrecord(NachrichenListe[ARow-1]).Data[i] >= 32 then
                Text[i+1] := chr(PCANrecord(NachrichenListe[ARow-1]).Data[i])
              else
                Text[i+1] := '.';
            end;
          end;
      end;
    end;
  end;
  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,Text);
end;

procedure TEmpfangForm.BerechneColWidth;
var
  i,w: integer;
begin
  if not TMainForm(Owner).actBinDaten.Checked then
    w := NachrichtenGrid.Canvas.TextWidth('$@@')
  else
    w := NachrichtenGrid.Canvas.TextWidth('000000000');

  if (NachrichtenGrid.ColWidths[2] = w) then
    NachrichtenGrid.Refresh
  else
    for i:=2 to 9 do
      NachrichtenGrid.ColWidths[i] := w;
end;

procedure TEmpfangForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
  tempTime: dword;
  tempAnzahl: dword;
  tempSumme: dword;
begin
  if ProtokollRadio.Checked then
  begin
    // Protokoll-Ansicht
    if NachrichenListe.Count = NachrichenListe.Capacity then
      NachrichenListe.Capacity := NachrichenListe.Capacity + 1024;
    FreeMsg := false;
    NachrichenListe.Add(CANrec);
    if TMainForm(owner).RxRefreshRate <= 0 then
    begin
      NachrichtenGrid.RowCount := NachrichenListe.Count + 1;
      NachrichtenGrid.Refresh;
    end;
  end
  else
  begin
    // ID-Listen-Ansicht
    i := 0;
    while (i < NachrichenListe.Count) do
    begin
      if PCANrecord(NachrichenListe[i]).ID = CANrec.ID then
      begin
        // Es wurde schon eine Nachricht mit dieser ID empfangen
        tempTime := PCANrecord(NachrichenListe[i]).Timestamp;
        tempAnzahl := PCANrecord(NachrichenListe[i]).MsgAnzahl;
        tempSumme := PCANrecord(NachrichenListe[i]).IntervalSumme;
        PCANrecord(NachrichenListe[i])^ := CANrec^;

        // Schutz vor berlauf von PCANrecord(NachrichenListe[i]).IntervalSumme
        if CANrec.Timestamp > tempTime then
        begin
          PCANrecord(NachrichenListe[i]).IntervalSumme := tempSumme + (CANrec.Timestamp - tempTime);
          PCANrecord(NachrichenListe[i]).MsgAnzahl := tempAnzahl + 1;
        end
        else
        begin
          if tempAnzahl = 0 then
            beep;
          PCANrecord(NachrichenListe[i]).IntervalSumme := tempSumme div tempAnzahl;
          PCANrecord(NachrichenListe[i]).MsgAnzahl := 1;
        end;

        if TMainForm(owner).RxRefreshRate <= 0 then
          NachrichtenGrid.Refresh;
        exit;
      end
      else
      begin
        if PCANrecord(NachrichenListe[i]).ID > CANrec.ID then
          // Da ID geordnet sind, muss nicht die gesamte Liste
          // durchsucht werden.
          break;
      end;

      inc(i);
    end;

    FreeMsg := false;
    CANrec.MsgAnzahl := 0;
    CANrec.IntervalSumme := 0;
    if i < NachrichenListe.Count then
      NachrichenListe.Insert(i, CANrec)
    else
      NachrichenListe.Add(CANrec);

    if TMainForm(owner).RxRefreshRate <= 0 then
    begin
      NachrichtenGrid.RowCount := NachrichenListe.Count + 1;
      NachrichtenGrid.Refresh;
    end;
  end;
end;

procedure TEmpfangForm.SpeichernBtnClick(Sender: TObject);
var
  i: integer;
  s: string;
  Datei: TextFile;
begin
  TMainForm(owner).SaveDialog.Title := 'CAN-Log speichern';
  TMainForm(owner).SaveDialog.DefaultExt := 'csv';
  TMainForm(owner).SaveDialog.FileName := TMainForm(owner).EmpfangDatei;
  TMainForm(owner).SaveDialog.Filter := 'CAN-Log (*.csv)|*.csv|Alle Dateien (*.*)|*.*';

  if TMainForm(owner).SaveDialog.Execute then
  begin
    TMainForm(owner).EmpfangDatei := TMainForm(owner).SaveDialog.FileName;
    try
      AssignFile(Datei, TMainForm(owner).SaveDialog.FileName);
      Rewrite(Datei);
      Writeln(Datei, 'ID;DLC;D0;D1;D2;D3;D4;D5;D6;D7;Rem.;Zeit;Interval');
      for i:=0 to NachrichenListe.Count-1 do
      begin
        s := format('%u;%u;%u;%u;%u;%u;%u;%u;%u;%u;%u',
              [PCANrecord(NachrichenListe[i]).ID,
               PCANrecord(NachrichenListe[i]).DLC,
               PCANrecord(NachrichenListe[i]).Data[0],
               PCANrecord(NachrichenListe[i]).Data[1],
               PCANrecord(NachrichenListe[i]).Data[2],
               PCANrecord(NachrichenListe[i]).Data[3],
               PCANrecord(NachrichenListe[i]).Data[4],
               PCANrecord(NachrichenListe[i]).Data[5],
               PCANrecord(NachrichenListe[i]).Data[6],
               PCANrecord(NachrichenListe[i]).Data[7],
               (PCANrecord(NachrichenListe[i]).Cmd shr 5) and $01 // Remote
              ]);
        if ProtokollRadio.Checked then
          s := s + format(';%u;;;',[PCANrecord(NachrichenListe[i]).Timestamp])
        else
          if (PCANrecord(NachrichenListe[i]).MsgAnzahl > 0) then
            s := s + format(';;%u;;',[PCANrecord(NachrichenListe[i]).IntervalSumme div PCANrecord(NachrichenListe[i]).MsgAnzahl])
          else
            s := s + ';;0';
        Writeln(Datei, s);
      end;
    finally
      CloseFile(Datei);
    end;
  end;
end;

procedure TEmpfangForm.RefreshTimerTimer(Sender: TObject);
begin
  if NachrichenListe.Count = 0 then
    NachrichtenGrid.RowCount := 2
  else
    NachrichtenGrid.RowCount := NachrichenListe.Count + 1;
  NachrichtenGrid.Refresh;
end;

procedure TEmpfangForm.SetColCount;
begin
  if (NachrichtenGrid.ColCount = 12) xor not TMainForm(owner).ASCIISpalteAktiv then
  begin
    if TMainForm(owner).ASCIISpalteAktiv then
    begin
      NachrichtenGrid.ColCount := 13;
      NachrichtenGrid.ColWidths[12] := 80;
    end
    else
      NachrichtenGrid.ColCount := 12;
  end;
end;

end.
