unit FTDI;

interface

uses
  Windows;

const
  FTDI_DLL = 'ftd2xx.dll';

  FT_OK = 0;
  FT_INVALID_HANDLE = 1;
  FT_DEVICE_NOT_FOUND = 2;
  FT_DEVICE_NOT_OPENED = 3;
  FT_IO_ERROR = 4;
  FT_INSUFFICIENT_RESOURCES = 5;
  FT_INVALID_PARAMETER = 6;

  // Flags for FT_OpenEx
  FT_OPEN_BY_SERIAL_NUMBER = 1;
  FT_OPEN_BY_DESCRIPTION = 2;
  FT_OPEN_BY_LOCATION = 4;

  // Flags for FT_ListDevices
  FT_LIST_NUMBER_ONLY = $80000000;
  FT_LIST_BY_INDEX = $40000000;
  FT_LIST_ALL = $20000000;

type
  TFtDeviceInfo = record
    Flags: dword;
    Typ: dword;
    PID: word;  // Teil von ID: dword;
    VID: word;  // Teil von ID: dword;
    LocId: dword;
    SerialNumber: array[0..15] of char;
    Description: array[0..63] of char;
    ftHandle: THandle;
  end;

  TFtProgramData = record
    Signature1: dword;      // Header - must be 0x00000000
    Signature2: dword;      // Header - must be 0xffffffff
    Version: dword;         // Header Version, 0 = original, 1 = FT2232C extensions, 2 = FT232R extensions
    VendorId: word;         // 0x0403
    ProductId: word;        // 0x6001
    Manufacturer: pchar;    // "FTDI"
    ManufacturerId: pchar;  // "FT"
    Description: pchar;     // "USB HS Serial Converter"
    SerialNumber: pchar;    // "FT000001" if fixed, or NULL
    MaxPower: word;         // 0 < MaxPower <= 500
    PnP: word;              // 0 = disabled, 1 = enabled
    SelfPowered: word;      // 0 = bus powered, 1 = self powered
    RemoteWakeup: word;     // 0 = not capable, 1 = capable

    // Rev4 extensions
    Rev4: byte;             // non-zero if Rev4 chip, zero otherwise
    IsoIn: byte;            // non-zero if in endpoint is isochronous
    IsoOut: byte;           // non-zero if out endpoint is isochronous
    PullDownEnable: byte;   // non-zero if pull down enabled
    SerNumEnable: byte;     // non-zero if serial number to be used
    USBVersionEnable: byte; // non-zero if chip uses USBVersion
    USBVersion: word;       // BCD (0x0200 => USB2)

    // FT2232C extensions
    Rev5: byte;             // non-zero if Rev5 chip, zero otherwise
    IsoInA: byte;           // non-zero if in endpoint is isochronous
    IsoInB: byte;           // non-zero if in endpoint is isochronous
    IsoOutA: byte;          // non-zero if out endpoint is isochronous
    IsoOutB: byte;          // non-zero if out endpoint is isochronous
    PullDownEnable5: byte;  // non-zero if pull down enabled
    SerNumEnable5: byte;    // non-zero if serial number to be used
    USBVersionEnable5: byte; // non-zero if chip uses USBVersion
    USBVersion5: word;      // BCD (0x0200 => USB2)
    AIsHighCurrent: byte;   // non-zero if interface is high current
    BIsHighCurrent: byte;   // non-zero if interface is high current
    IFAIsFifo: byte;        // non-zero if interface is 245 FIFO
    IFAIsFifoTar: byte;     // non-zero if interface is 245 FIFO CPU target
    IFAIsFastSer: byte;     // non-zero if interface is Fast serial
    AIsVCP: byte;           // non-zero if interface is to use VCP drivers
    IFBIsFifo: byte;        // non-zero if interface is 245 FIFO
    IFBIsFifoTar: byte;     // non-zero if interface is 245 FIFO CPU target
    IFBIsFastSer: byte;     // non-zero if interface is Fast serial
    BIsVCP: byte;           // non-zero if interface is to use VCP drivers

    // FT232R extensions
    UseExtOsc: byte;        // Use External Oscillator
    HighDriveIOs: byte;     // High Drive I/Os
    EndpointSize: byte;     // Endpoint size
    PullDownEnableR: byte;  // non-zero if pull down enabled
    SerNumEnableR: byte;    // non-zero if serial number to be used
    InvertTXD: byte;        // non-zero if invert TXD
    InvertRXD: byte;        // non-zero if invert RXD
    InvertRTS: byte;        // non-zero if invert RTS
    InvertCTS: byte;        // non-zero if invert CTS
    InvertDTR: byte;        // non-zero if invert DTR
    InvertDSR: byte;        // non-zero if invert DSR
    InvertDCD: byte;        // non-zero if invert DCD
    InvertRI: byte;         // non-zero if invert RI
    Cbus0: byte;            // Cbus Mux control - Ignored for FT245R
    Cbus1: byte;            // Cbus Mux control - Ignored for FT245R
    Cbus2: byte;            // Cbus Mux control - Ignored for FT245R
    Cbus3: byte;            // Cbus Mux control - Ignored for FT245R
    Cbus4: byte;            // Cbus Mux control - Ignored for FT245R
    RIsVCP: byte;           // non-zero if using VCP drivers
  end;
  pFtProgramData =  ^TFtProgramData;

{ FTDI Funktionen }

  function FT_Open(PVDevice:Integer; ftHandle:Pointer ) : integer ; stdcall ; External FTDI_DLL name 'FT_Open';
  function FT_Close(ftHandle:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_Close';
  function FT_Read(ftHandle:Dword; FTInBuf : Pointer; BufferSize : LongInt; ResultPtr : Pointer ) : integer ; stdcall ; External FTDI_DLL name 'FT_Read';
  function FT_Write(ftHandle:Dword; FTOutBuf : Pointer; BufferSize : LongInt; ResultPtr : Pointer ) : integer ; stdcall ; External FTDI_DLL name 'FT_Write';
  function FT_SetBaudRate(ftHandle:Dword;BaudRate:DWord) : integer ; stdcall ; External FTDI_DLL name 'FT_SetBaudRate';
  function FT_SetDataCharacteristics(ftHandle:Dword;WordLength,StopBits,Parity:Byte) : integer ; stdcall ; External FTDI_DLL name 'FT_SetDataCharacteristics';
  function FT_SetFlowControl(ftHandle:Dword;FlowControl:Word;XonChar,XoffChar:Byte) : integer ; stdcall ; External FTDI_DLL name 'FT_SetFlowControl';
  function FT_ResetDevice(ftHandle:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_ResetDevice';
  function FT_SetChars(ftHandle:Dword;EventChar,EventCharEnabled,ErrorChar,ErrorCharEnabled : Byte) : integer ; stdcall ; External FTDI_DLL name 'FT_SetChars';
  function FT_Purge(ftHandle:Dword;Mask:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_Purge';
  function FT_SetTimeouts(ftHandle:Dword;ReadTimeout,WriteTimeout:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_SetTimeouts';
  function FT_GetQueueStatus(ftHandle:Dword;RxBytes:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_GetQueueStatus';
  function FT_ListDevices(pvArg1:Pointer;pvArg2:Pointer;dwFlags:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_ListDevices';
  function FT_OpenEx(pvArg1:Pointer;dwFlags:Dword;ftHandle:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_OpenEx';

  function FT_GetStatus(ftHandle:Dword;pvArg1:Pointer;pvArg2:Pointer;pvArg3:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_GetStatus';
  function FT_CreateDeviceInfoList(pvArg1:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_CreateDeviceInfoList';
  function FT_GetDeviceInfoList(pvArg1:Pointer;pvArg2:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_GetDeviceInfoList';
  function FT_GetDriverVersion(ftHandle:Dword;pvArg1:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_GetDriverVersion';
  function FT_GetLibraryVersion(pvArg1:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_GetLibraryVersion';
  function FT_SetEventNotification(ftHandle:Dword;dwEventMask:Dword;pvArg1:pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_SetEventNotification';
//FT_SetLatencyTimer(FT_HANDLE ftHandle, UCHAR ucTimer)
//FT_SetUSBParameters(FT_HANDLE ftHandle, DWORD dwInTransferSize, DWORD dwOutTransferSize)

  function FT_ReadEE(ftHandle:Dword;pvArg1:Dword;pvArg2:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_ReadEE';
  function FT_EE_Read(ftHandle:Dword;pvArg1:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_EE_Read';
  function FT_EE_Program(ftHandle:Dword;pvArg1:Pointer) : integer ; stdcall ; External FTDI_DLL name 'FT_EE_Program';
  function FT_Rescan() : integer ; stdcall ; External FTDI_DLL name 'FT_Rescan';
  function FT_ResetPort(ftHandle:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_ResetPort';
  function FT_CyclePort(ftHandle:Dword) : integer ; stdcall ; External FTDI_DLL name 'FT_CyclePort';

{ Funktionen nicht in der DLL }

function FT_GetNumDevices(DeviceNum: pdword): integer;

implementation

function FT_GetNumDevices(DeviceNum: pdword): integer;
var
  dummy: pdword;
begin
  dummy := nil;
  result := FT_ListDevices(DeviceNum, dummy, FT_LIST_NUMBER_ONLY);
end;

end.
