unit Firmware;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, StrUtils, ExtCtrls, FTDI;

const
  FLASHMODUS_WRITE = 1;
  FLASHMODUS_VERIFY = 2;
  FLASHSTART_ADR = $4000;
  DONGLE_BESCHREIBUNG = 'CAN-Interceptor';

type
  IDArray = array[0..6] of byte;

  TFirmwareForm = class(TForm)
    SchliessenBtn: TBitBtn;
    OeffnenBtn: TBitBtn;
    IDGroup: TGroupBox;
    ID0Edit: TEdit;
    ID1Edit: TEdit;
    ID2Edit: TEdit;
    ID3Edit: TEdit;
    ID4Edit: TEdit;
    ID5Edit: TEdit;
    ID6Edit: TEdit;
    LoeschenBtn: TBitBtn;
    FlashBtn: TBitBtn;
    VerifyBtn: TBitBtn;
    AutoBtn: TBitBtn;
    CheckIDBtn: TBitBtn;
    Label1: TLabel;
    DateiNameEdit: TEdit;
    Label2: TLabel;
    FlashMemo: TMemo;
    Bevel1: TBevel;
    BlackCheckBtn: TBitBtn;
    Bevel2: TBevel;
    SerienNrBeschriftungLabel: TLabel;
    SerienNrComboBox: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure CheckIDBtnClick(Sender: TObject);
    procedure OeffnenBtnClick(Sender: TObject);
    procedure LoeschenBtnClick(Sender: TObject);
    procedure IDEditChange(Sender: TObject);
    procedure IDEditEnter(Sender: TObject);
    procedure IDEditKeyPress(Sender: TObject; var Key: Char);
    procedure AutoBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BlackCheckBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure SerienNrComboBoxDropDown(Sender: TObject);
  private
    { Private-Deklarationen }
    hDongle: THandle;
    hDongleRxEvent: THandle;
    BaudRateIdx: shortint;
    DongleActive: boolean;

    MotDatei: TextFile;

    function OpenPort(Sender: TObject): integer;
    function InitChip(): boolean;
    procedure CheckID();
    function ClearStatus(): integer;
    function GetStatus(Status: pword; TimeOut: dword = 20): integer;
    function GetVersion(): string;
    function EraseBlock(Adresse: longword): word;
    function EraseChip(): word;
    function WritePage(Page: word; Data: array of byte): word;
    function ReadPage(Page: word; ReadBuf: pbyte): integer;
    function FlashFile(Modus: byte): integer;
    function BlankCheck(): integer;
    function GetMot2Adresse(Text: string; Checksum: pbyte): integer;
    function GetMot2Data(Text: string; ByteNr: integer; Checksum: pbyte): integer;
    function GetMot2DataLength(Text: string; Checksum: pbyte): integer;

    procedure SetControlsStatus(Status: boolean = false);
    procedure ChangeFTDIData();
  public
    { Public-Deklarationen }
  end;

var
  IdAdressen: array[0..6] of integer = ($0ffdf,$0ffe3,$0ffeb,$0ffef,$0fff3,$0fff7,$0fffb);
  Baud: array[0..4] of integer = (9600,19200,38400,57600,115200);

implementation

uses Hauptfenster;

{$R *.dfm}

procedure TFirmwareForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  TMainForm(owner).ButtonImages.GetBitmap(5, OeffnenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(8, SchliessenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(6, FlashBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(11, AutoBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(10, CheckIDBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(1, LoeschenBtn.Glyph);

  for i := 0 to IDGroup.ControlCount - 1 do
  begin
    if (IDGroup.Controls[i] is TEdit) and (LowerCase(LeftStr(IDGroup.Controls[i].Name, 2)) = 'id') then
    begin
      TEdit(IDGroup.Controls[i]).Tag := 255;
      IDEditEnter(TEdit(IDGroup.Controls[i]));
    end;
  end;

  SetControlsStatus(false);
  BaudRateIdx := -1;
  hDongleRxEvent := CreateEvent(nil,True,False,'');
end;

procedure TFirmwareForm.FormShow(Sender: TObject);
var
  DongleAnzahl: dword;
  pcDongleSerNr: array[0..8] of char;
begin
  if SerienNrComboBox.Text = '' then
  begin
    FT_GetNumDevices(@DongleAnzahl);
    if DongleAnzahl > 0 then
    begin
      FT_ListDevices(pointer(0), @pcDongleSerNr, FT_LIST_BY_INDEX or FT_OPEN_BY_SERIAL_NUMBER);
      SerienNrComboBox.Text := pcDongleSerNr;
    end;
  end;
end;

procedure TFirmwareForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if DongleActive then
    Action := caNone;
end;

procedure TFirmwareForm.FormDestroy(Sender: TObject);
begin
  CloseHandle(hDongleRxEvent);
end;

procedure TFirmwareForm.SetControlsStatus(Status: boolean = false);
var
  i: integer;
begin
  if DongleActive <> Status then
  begin
    DongleActive := Status;
    for i:=0 to self.ControlCount-1 do
    begin
      if self.Controls[i] is TBitBtn then
        self.Controls[i].Enabled := not Status;
    end;
  end;
end;

procedure TFirmwareForm.OeffnenBtnClick(Sender: TObject);
var
  IDDatei: TextFile;
  IDZeile: string;
  DateiOpenFlag: boolean;
  IDEdit: TEdit;
  ValueN, ValueB: byte;
  i: integer;
begin
  DateiOpenFlag := false;
  ValueB := 0;

  TMainForm(Owner).OpenDialog.DefaultExt := '*.mot';
  TMainForm(Owner).OpenDialog.Filter := 'Motoroler S2 (*.mot)|*.mot|Alle Dateien (*.*)|*.*';
  TMainForm(Owner).OpenDialog.Options := TMainForm(Owner).OpenDialog.Options + [ofFileMustExist,ofPathMustExist];
  if TMainForm(Owner).OpenDialog.Execute then
  begin
    DateiNameEdit.Text := TMainForm(Owner).OpenDialog.FileName;
    try
      // Datei ffnen, und Zeiger auf Anfang der Datei setzen
      AssignFile(IDDatei, ChangeFileExt(DateiNameEdit.Text, '.id'));
      Reset(IDDatei);
      DateiOpenFlag := true;

      // Erste Zeile lesen und berprfen
      Readln(IDDatei, IDZeile);
      if (LeftStr(IDZeile, 4) = '-ID#') and (Length(IDZeile) >= 18) then
      begin
        for i := 0 to 13 do
        begin
          case IDZeile[i+5] of
            '0'..'9': ValueN := ord(IDZeile[i+5]) - ord('0');
            'a'..'f','A'..'F': ValueN := (ord(IDZeile[i+5]) and $df) - ord('A') + 10;
          else
            break;
          end;
          if (i and $01) = 0 then
            ValueB := ValueN shl 4
          else
          begin
            IDEdit := TEdit(self.FindComponent('ID'+IntToStr(i shr 1)+'Edit'));
            IDEdit.Tag := ValueB + ValueN ;
            IDEditEnter(IDEdit);
          end;
        end;
      end;
    finally
      if DateiOpenFlag then
        CloseFile(IDDatei);
    end;
  end;
end;

procedure TFirmwareForm.IDEditChange(Sender: TObject);
begin
  TMainForm(Owner.Owner).CheckZahlenTEditEingabe(TEdit(Sender), $ff);
end;

procedure TFirmwareForm.IDEditEnter(Sender: TObject);
begin
  TEdit(Sender).Text := '$' + RightStr(format('00%x',[TEdit(Sender).Tag]),2);
end;

procedure TFirmwareForm.IDEditKeyPress(Sender: TObject; var Key: Char);
begin
  TMainForm(owner).HexEditKeyPress(Sender, Key);
end;

function TFirmwareForm.GetMot2Adresse(Text: string; Checksum: pbyte): integer;
var
  i: integer;
  j: integer;
begin
  result := 0;
  for i := -3 to -1 do
  begin
    j := GetMot2Data(Text, i, Checksum);
    if j < 0 then
    begin
      result := j;
      exit;
    end;
    result := (result shl 8) + j;
  end;
  result := result and $00ffffff;
end;

function TFirmwareForm.GetMot2DataLength(Text: string; Checksum: pbyte): integer;
begin
  result := GetMot2Data(Text, -4, Checksum) - 3;
end;

function TFirmwareForm.GetMot2Data(Text: string; ByteNr: integer; Checksum: pbyte): integer;
var
  i: integer;
  p: integer;
begin
  p := 11 + (2 * ByteNr);
  if (p <= 2) or (p > Length(Text) - 1) then
  begin
    result := -1;
    exit;
  end;
  i := ord(Text[p]);
  case i of
    $30..$39: result := (i and $0f) shl 4;
    $41..$46: result := ((i - $37) and $0f) shl 4;
    $61..$66: result := ((i - $57) and $0f) shl 4;
  else
    result := -1;
    exit;
  end;
  i := ord(Text[p + 1]);
  case i of
    $30..$39: result := result + (i and $0f);
    $41..$46: result := result + ((i - $37) and $0f);
    $61..$66: result := result + ((i - $57) and $0f);
  else
    result := -1;
    exit;
  end;
  Checksum^ := Checksum^ + result;
end;

function TFirmwareForm.OpenPort(Sender: TObject): integer;
var
  DongleAnzahl: dword;
  pcDongleSerNr: array[0..8] of char;
  pcDongleBeschreibung: array[0..40] of char;
  tempstr: string;
begin
  result := FT_GetNumDevices(@DongleAnzahl);
  case DongleAnzahl of
    0:  begin
          MessageDlg('Es sind keine CAN-Dongle angeschlossen.', mtError, [mbOK], 0);
          result := -1;
          exit;
        end;
    1:  begin
          SerienNrComboBox.Items.Clear;
          FT_ListDevices(pointer(0), @pcDongleSerNr, FT_LIST_BY_INDEX or FT_OPEN_BY_SERIAL_NUMBER);
          FT_ListDevices(pointer(0), @pcDongleBeschreibung, FT_LIST_BY_INDEX or FT_OPEN_BY_DESCRIPTION);
          SerienNrComboBox.Items.Add(pcDongleSerNr + '; ' + pcDongleBeschreibung);
          SerienNrComboBox.ItemIndex := 0;
        end;
  else
    if SerienNrComboBox.Text = '' then
    begin
      MessageDlg('Bitte zuerst einen CAN-Dongle answhlen.', mtError, [mbOK], 0);
      exit;
    end;
  end;

  // Schnittstelle ffnen
  tempstr := copy(SerienNrComboBox.Text, 0, pos('; ', SerienNrComboBox.Text) - 1);
  result := FT_OpenEx(pchar(tempstr),
        FT_OPEN_BY_SERIAL_NUMBER, @hDongle);
//  result := FT_Open(0, @hDongle);
  If result <> FT_OK then
  begin
    MessageDlg(format('Fehler %d beim ffnen der Schnittstelle.', [result] ), mtError, [mbOK], 0);
    exit;
  end;

  // Schnittstellenparameter setzen
  result := FT_SetFlowControl(hDongle, 0, 0, 0); // FT_FLOW_NONE
  result := result or FT_SetBaudRate(hDongle, 9600);
  result := result or FT_SetDataCharacteristics(hDongle, 8, 0, 0); // 8N1
  result := result or FT_SetEventNotification(hDongle, 1, pointer(hDongleRxEvent)); // FT_EVENT_RXCHAR
  If result <> FT_OK then
  begin
    MessageDlg(format('Fehler %d beim Setzten der Schnittstellenparameter.', [result] ), mtError, [mbOK], 0);
    exit;
  end;

  // Buffer lschen
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
end;

function TFirmwareForm.InitChip(): boolean;
var
  Command: byte;
  Receive: byte;
  LastCommand: byte;
  RxByteInQueue: integer;
  RxBytes: integer;
  TxBytes: integer;
  i: integer;
  tempbool: boolean;
  Status: word;
  label Wiederholung;
begin
Wiederholung:
  result := false;
  tempbool := false;

  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX

  if BaudRateIdx < 0 then
  begin
    // MC mit standard Baud-Rate initialisieren
    BaudRateIdx := 0;
    Command := $b0 + BaudRateIdx;
    FT_SetBaudRate(hDongle, Baud[BaudRateIdx]);
    FT_Write(hDongle, @Command, 1, @TxBytes);
    sleep(50);

    Command := $00;
    for i:=0 to 15 do
    begin
      FT_Write(hDongle, @Command, 1, @TxBytes);
      sleep(20);
    end;
    Command := $b0 + BaudRateIdx;
  end
  else
  begin
    Command := $b0 + BaudRateIdx;
    FT_SetBaudRate(hDongle, Baud[BaudRateIdx]);
  end;

  FT_Write(hDongle, @Command, 1, @TxBytes);
  sleep(100);

  // Anzahl der empfangenen Bytes ermitteln
  FT_GetQueueStatus(hDongle,@RxByteInQueue);
  if RxByteInQueue > 0 then
  begin
    FT_Read(hDongle , @Receive, 1, @RxBytes);
    if Receive = Command then
      tempbool := true
    else
      sleep(200);
  end
  else
    sleep(200);

  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX

  // Prfen, ob MC schon mit anderer Baud-Rate initialisiert ist
  if not tempbool then
  begin
    Command := $b1;
    repeat
      if FT_SetBaudRate(hDongle, Baud[Command and $0f]) <> FT_OK then
      begin
        MessageDlg('Fehler bei setzten der Schnittstellenparameter!', mtError, [mbOK], 0);
        exit;
      end;

      for i:=0 to 15 do
      begin
        FT_Write(hDongle, @Command, 1, @TxBytes);
        sleep(20);
      end;

      ResetEvent(hDongleRxEvent);
      FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
      FT_Write(hDongle, @Command, 1, @TxBytes);
//      sleep(50);
      WaitForSingleObject(hDongleRxEvent, 50);

      FT_GetQueueStatus(hDongle,@RxByteInQueue);
      if RxByteInQueue > 0 then
      begin
        FT_Read(hDongle , @Receive, 1, @RxBytes);
        if Receive = Command then
        begin
          tempbool := true;
          break;
        end;
      end;

      command := command + 1;
    until tempbool or (Command = $b5);
    FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
  end;

  LastCommand := Command;

  // Prfen, ob MC mit einer hheren Baud-Rate initialisiert werden kann
  if (Command < $b4) then
  begin
    while ((Command <= $b4) and tempbool) do
    begin
      ResetEvent(hDongleRxEvent);
      FT_Write(hDongle, @Command, 1, @TxBytes);
//      sleep(50);
      WaitForSingleObject(hDongleRxEvent, 50);

      FT_GetQueueStatus(hDongle,@RxByteInQueue);
      if RxByteInQueue = 0 then
      begin
        Command := Command - 1;
        break;
      end;

      FT_Read(hDongle , @Receive, 1, @RxBytes);
      if Receive <> Command then
      begin
        Command := Command - 1;
        break;
      end;

      if (Command = $b4) then
        break;

      inc(Command);

      ResetEvent(hDongleRxEvent);
      FT_Write(hDongle, @Command, 1, @TxBytes);
//      sleep(50);
      WaitForSingleObject(hDongleRxEvent, 50);

      FT_GetQueueStatus(hDongle,@RxByteInQueue);
      if RxByteInQueue = 0 then
      begin
        Command := Command - 1;
        break;
      end;

      FT_Read(hDongle , @Receive, 1, @RxBytes);
      if Receive <> Command then
      begin
        Command := Command - 1;
        break;
      end;

      LastCommand:=Command;

      if FT_SetBaudRate(hDongle, Baud[Command and $0f]) <> FT_OK then
      begin
        MessageDlg('Fehler bei setzten der Schnittstellenparameter!', mtError, [mbOK], 0);
        exit;
      end;
    end;

    FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
    sleep(200);
  end;

  if LastCommand <> Command then
  begin
    if FT_SetBaudRate(hDongle, Baud[Command and $0f]) <> FT_OK then
    begin
      MessageDlg('Fehler bei setzten der Schnittstellenparameter!', mtError, [mbOK], 0);
      exit;
    end;

    Command := $00;
    for i:=0 to 15 do
    begin
      FT_Write(hDongle, @Command, 1, @TxBytes);
      sleep(20);
    end;
    Command := LastCommand;
    FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
    sleep(200);
  end;

  if not tempbool then
  begin
    beep;
    if MessageDlg('Fehler beim initialisieren des MCs!'+#13#10+
          'Bitte berprfe die Verbindung und den Jumper fr'+#13#10+
          'den Flash-Modus.',mtError,[mbOK,mbCancel],0) = mrOK then
    begin
      goto Wiederholung;
    end;
    FlashMemo.Lines.Add('Initialisierung abgebroch.');
    exit;
  end;

  BaudRateIdx := LastCommand and $0f;
  FlashMemo.Lines.Add(format('Initialisierung mit %d Baud OK.', [Baud[BaudRateIdx]]));
  FlashMemo.Lines.Add('Bootloader: '+ GetVersion());

  // Statusregister lschen
  ClearStatus();

  // Statusregister lesen
  GetStatus(@Status);
//  LogRichEdit.Lines.Add(format('Status: %x',[Status]));
  if (Status and $0c) <> $0c then
  begin
    CheckID();
    GetStatus(@Status);
  end;

  if (Status and $0c) <> $0c then
  begin
    beep;
    FlashMemo.Lines.Add('ID falsch!');
    exit;
  end;

  FlashMemo.Lines.Add(format('ID korrekt. Status: %x',[Status]));
  result := true;
end;

{---------------------------------------------------------
// Statusregister lesen
// Bit 0 - DataNum (over/right)
// Bit 1 - Timeout (timeout/right)
// Bit 2 - ID Check, ID wurde gesendet
// Bit 3 - ID Check, ID korrekt
// Bit 4 - Checksum (match/ not match)
// Bit 12 - After Block (Error/right)
// Bit 13 - Programm (error/right)
// Bit 14 - Erase (error/right)
// Bit 15 - Machine state (ready/busy)
----------------------------------------------------------}
function TFirmwareForm.GetStatus(Status: pword; TimeOut: dword = 20): integer;
var
  Command: byte;
  Sended, RxByteInQueue, Readed: dword;
  StatusBuf: array[0..1] of byte;
begin
  result := 0;
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX

  Command := $70;
  ResetEvent(hDongleRxEvent);
  FT_Write(hDongle, @Command, 1, @Sended);
//  sleep(50);
  if WaitForSingleObject(hDongleRxEvent, TimeOut) <> WAIT_OBJECT_0 then
    FlashMemo.Lines.Add('GetStatus Timeout');

  FT_GetQueueStatus(hDongle,@RxByteInQueue);
  FT_Read(hDongle , @StatusBuf, RxByteInQueue, @Readed);
  if Readed <> 2 then
  begin
    result := -1;
    Status^ := 0;
  end
  else
    Status^ := (StatusBuf[0] shl 8) or StatusBuf[1];
end;

{---------------------------------------------------------
// Status lschen
----------------------------------------------------------}
function TFirmwareForm.ClearStatus(): integer;
var
  Command: byte;
  Sended: dword;
begin
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
  Command := $50;
  result := FT_Write(hDongle, @Command, 1, @Sended);
  sleep(20);
end;

{---------------------------------------------------------
// ID Senden
----------------------------------------------------------}
procedure TFirmwareForm.CheckID();
var
  i: integer;
  SendBuffer: array[0..11] of byte;
  Sended: dword;
  IDEdit: TEdit;
begin
  SendBuffer[0]  := $f5;   // ID senden Command
  SendBuffer[1]  := $df;   // Adresse der ID (lo)
  SendBuffer[2]  := $ff;   // Adresse der ID (mid)
  SendBuffer[3]  := $00;   // Adresse der ID (hi)
  SendBuffer[4]  := $07;   // Lnge der ID

  for i := 0 to 6 do
  begin
    IDEdit := TEdit(self.FindComponent('ID'+IntToStr(i)+'Edit'));
    if assigned(IDEdit) then
      SendBuffer[5 + i] := IDEdit.Tag;
  end;

  FT_Write(hDongle, @SendBuffer, 12, @Sended);
  sleep(50);
end;

{---------------------------------------------------------
// Status lschen
----------------------------------------------------------}
function TFirmwareForm.GetVersion(): string;
var
  Command: byte;
  Sended, Readed, RxByteInQueue: dword;
  ReadBuf: array[0..9] of char;
begin
  FillChar(result, 10, 0);
  ResetEvent(hDongleRxEvent);

  Command := $fb;
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
  FT_Write(hDongle, @Command, 1, @Sended);

  if WaitForSingleObject(hDongleRxEvent, 50) <> WAIT_OBJECT_0 then
  begin
    FlashMemo.Lines.Add('Zeitberschreitung!');
    exit;
  end;
  ResetEvent(hDongleRxEvent);

  FT_GetQueueStatus(hDongle,@RxByteInQueue);
  if RxByteInQueue > 8 then
    RxByteInQueue := 8;
  FT_Read(hDongle , @ReadBuf, RxByteInQueue, @Readed);
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX

  ReadBuf[Readed] := #0;
  result := ReadBuf;
end;

{---------------------------------------------------------
// Gesamten Flash lschen
----------------------------------------------------------}
function TFirmwareForm.EraseChip(): word;
var
  CommandBuf: array[0..1] of byte;
  Sended: dword;
  Status: word;
begin
  ClearStatus();

  CommandBuf[0] := $75;
  FT_Write(hDongle, @CommandBuf, 1, @Sended);
  sleep(20);

  CommandBuf[0] := $a7;
  CommandBuf[1] := $d0;
  FT_Write(hDongle, @CommandBuf, 2, @Sended);
  sleep(200);

  GetStatus(@Status, 15000);
  result := Status;
end;

{---------------------------------------------------------
// Einen einzelnen Flash-Block lschen
----------------------------------------------------------}
function TFirmwareForm.EraseBlock(Adresse: longword): word;
var
  CommandBuf: array[0..3] of byte;
  Sended: dword;
  Status: word;
begin
  ClearStatus();

  CommandBuf[0] := $75;
  FT_Write(hDongle, @CommandBuf, 1, @Sended);
  sleep(20);

  CommandBuf[0] := $20;
  CommandBuf[1] := (Adresse shr 8) and $ff;
  CommandBuf[2] := (Adresse shr 16) and $ff;
  CommandBuf[3] := $d0;
  FT_Write(hDongle, @CommandBuf, 4, @Sended);
  sleep(20);

  GetStatus(@Status);
  result := Status;
end;

{---------------------------------------------------------
// Flash Seite schreiben
----------------------------------------------------------}
function TFirmwareForm.WritePage(Page: word; Data: array of byte): word;
var
  CommandBuf: array[0..2] of byte;
  Sended: dword;
  Status: word;
begin
  ClearStatus();

  CommandBuf[0] := $75;
  FT_Write(hDongle, @CommandBuf, 1, @Sended);
  sleep(20);

  CommandBuf[0] := $41;
  CommandBuf[1] := Page and $ff;
  CommandBuf[2] := (Page shr 8) and $ff;
  FT_Write(hDongle, @CommandBuf, 3, @Sended);
  FT_Write(hDongle, @Data, 256, @Sended);
  sleep(20);

  GetStatus(@Status);
  result := Status;
end;

function TFirmwareForm.ReadPage(Page: word; ReadBuf: pbyte): integer;
var
  CommandBuf: array[0..2] of byte;
  RxByteInQueue, Readed, Sended: integer;
  ByteCount: integer;
begin
  result := 0;
  FT_Purge(hDongle, 3); // FT_PURGE_RX + FT_PURGE_TX
  FillChar(ReadBuf^, 256, $ff);
  ResetEvent(hDongleRxEvent);

  CommandBuf[0] := $ff;
  CommandBuf[1] := Page and $ff;
  CommandBuf[2] := (Page shr 8) and $ff;
  FT_Write(hDongle, @CommandBuf, 3, @Sended);

  ByteCount := 256;
  while (ByteCount > 0) do
  begin
    if WaitForSingleObject(hDongleRxEvent, 100) <> WAIT_OBJECT_0 then
    begin
      MessageDlg('Zeitberschreitung beim lesen!', mtError, [mbOK], 0);
      ResetEvent(hDongleRxEvent);
      result := -1;
      break;
    end;
    ResetEvent(hDongleRxEvent);

    FT_GetQueueStatus(hDongle,@RxByteInQueue);
    if RxByteInQueue > 0 then
    begin
      FT_Read(hDongle , ReadBuf, RxByteInQueue, @Readed);
      inc(ReadBuf, Readed);
      ByteCount := ByteCount - Readed;
    end;
  end;
end;

function TFirmwareForm.BlankCheck: integer;
var
  i: integer;
  PageNr: integer;     // Nummer der aktuellen Page
  PageBuffer: array[0..255] of byte;
begin
  result := 0;
  PageNr := FLASHSTART_ADR shr 8;

  while (PageNr <= $ff) and (result = 0) do
  begin
    if ReadPage(PageNr, @PageBuffer) <> 0 then
      exit;
    FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
          format('Prfe Page: %0.4x', [PageNr]);
    for i := 0 to 255 do
    begin
      if PageBuffer[i] <> $ff then
      begin
        result := (PageNr shl 8) + i;
        break;
      end;
    end;
    inc(PageNr);
  end;
  if result = 0 then
    FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] := 'Flash ist leer.     '
  else
    FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
              format('Adresse %0.4x ist schon beschrieben.', [result]);
end;

function TFirmwareForm.FlashFile(Modus: byte): integer; // Modus: Bit0-Write, Bit1-Verify
var
  Status: word;
  i: integer;

  MotZeile: string;
  PageBuffer: array[0..255] of byte;
  ReadBuffer: array[0..255] of byte;
  Zeile: integer;
  PageNr: integer;     // Nummer der aktuellen Page
  PageAdr: integer;    // ByteAdresse in der Page
  DatenBytes: integer;
  ByteCount: integer;
  CheckSum: byte;
  ExitFlag: boolean;
begin
  result := 0;

  // Zwischenspeicher initialisieren
  FillChar(PageBuffer, 256, $ff);
  PageNr := 0;     // Nummer der aktuellen Page
  ExitFlag := false;
  Zeile := 0;

  // Zeilen Schleife
  repeat
    CheckSum := 0;
    Readln(MotDatei, MotZeile);
    inc(Zeile);

    // Datei Ende ?
    if (LeftStr(MotZeile,2) = 'S8') then
      ExitFlag := true
    else
    begin
      // Normale Mot2 Datenzeile?
      if (LeftStr(MotZeile, 2) <> 'S2') then
      begin
        MessageDlg('Datei ist nicht vom Typ MOT2!', mtError, [mbCancel], 0);
        result := -1;
        exit;
      end;
    end;

    PageAdr := GetMot2Adresse(MotZeile, @CheckSum);
    if (PageNr = 0) then
      PageNr := PageAdr shr 8;
    PageAdr := PageAdr - (PageNr shl 8);

    ByteCount := 0;
    DatenBytes := GetMot2DataLength(MotZeile, @CheckSum);
    while (ByteCount < DatenBytes) do
    begin
      if (PageAdr >= $100) or (PageAdr < 0) then
      begin
        // Sol gerade der Block mit der ID geschrieben werden
        if (PageNr = $00ff) then
        begin
          // ID setzen
          for i := 0 to 6 do
          begin
//              PageBuffer[IdAdressen[i] and $ff] := ID[i];
            PageBuffer[IdAdressen[i] and $ff] := $ff;
          end;
        end;
        if (Modus and $01) > 0 then
        begin
          status := WritePage(PageNr, PageBuffer);
          FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
              format('Schreibe Page: %0.4x, Status: %0.4x',[PageNr, status]);
        end;
        if (Modus and $02) > 0 then
        begin
          if (Modus and $01) = 0 then
            FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
                format('Prfe Page: %0.4x',[PageNr]);

          if ReadPage(PageNr, @ReadBuffer) <> 0 then
          begin
            result := -1;
            exit;
          end;
          for i := 0 to 255 do
          begin
            if PageBuffer[i] <> ReadBuffer[i] then
            begin
              FlashMemo.Lines.Add(
                format('Adresse %0.4x ist fehlerhaft.',[(PageNr shl 8) + i]));
              result := -1;
              exit;
            end;
          end;
        end;

        FlashMemo.Refresh;
        FillChar(PageBuffer, 256, $ff);
        if (ByteCount < DatenBytes - 1) then
        begin
          PageNr := PageNr + (PageAdr shr 8);
          PageAdr := PageAdr and $ff;
        end
        else
        begin
          PageNr := 0;
          PageAdr := 0;    // ByteAdresse in der Page
        end;
      end;

      if (ByteCount < DatenBytes - 1) then
      begin
        // Datenbyte lesen
        PageBuffer[PageAdr] := GetMot2Data(MotZeile, ByteCount, @CheckSum);
        inc(PageAdr);
      end
      else
      begin
        // Prfsumme lesen
        GetMot2Data(MotZeile, ByteCount, @CheckSum);
        if (CheckSum <> $ff) then
        begin
          FlashMemo.Lines.Add(format('Prfsummenfehler in Zeile %d', [Zeile]));
          FlashMemo.Lines.Add('');
          beep;
        end;
      end;
      inc(ByteCount);
    end;
  until (ExitFlag or Eof(MotDatei));  // Zeilen Schleife
end;

procedure TFirmwareForm.CheckIDBtnClick(Sender: TObject);
begin
  FlashMemo.Lines.Add('Schreibschutz prfen');
  try
    SetControlsStatus(true);

    // Schnittstelle ffnen
    if OpenPort(Sender) <> FT_OK then
      exit;
    InitChip();
  finally
    // Schnittstelle schlieen
    FT_Close(hDongle);
    FlashMemo.Lines.Add('------------------------------------');
    SetControlsStatus(false);
  end;
end;

procedure TFirmwareForm.LoeschenBtnClick(Sender: TObject);
begin
  FlashMemo.Lines.Add('Flash-Speicher lschen');
  try
    SetControlsStatus(true);

    // Schnittstelle ffnen
    if OpenPort(Sender) <> FT_OK then
      exit;

    FlashMemo.Lines.Add('Flash-Speicher wird gelschen');
    if InitChip() then
    begin
      FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
                format('Chip gelscht. Status: %0.4x     ', [EraseChip()]);
    end;
  finally
    // Schnittstelle schlieen
    FT_Close(hDongle);
    FlashMemo.Lines.Add('------------------------------------');
    SetControlsStatus(false);
  end;
end;

procedure TFirmwareForm.BlackCheckBtnClick(Sender: TObject);
begin
  try
    SetControlsStatus(true);

    // Schnittstelle ffnen
    if OpenPort(Sender) <> FT_OK then
      exit;
    if InitChip() then
      BlankCheck();
  finally
    // Schnittstelle schlieen
    FT_Close(hDongle);
    FlashMemo.Lines.Add('------------------------------------');
    SetControlsStatus(false);
  end;
end;

procedure TFirmwareForm.AutoBtnClick(Sender: TObject);
var
  Status: integer;
  DateiOpenFlag: boolean;
  MotZeile: string;
begin
  if TEdit(sender).Name = 'AutoBtn' then
    FlashMemo.Lines.Add('Leertest, Lschen, Programmieren und Prfen');
  if TEdit(sender).Name = 'FlashBtn' then
    FlashMemo.Lines.Add('Flash-Speicher programmieren');
  if TEdit(sender).Name = 'VerifyBtn' then
    FlashMemo.Lines.Add('Auf bereinstimmung prfen');

  DateiOpenFlag := false;

  try
    if DateiNameEdit.Text = '' then
    begin
      MessageDlg('Keine Datei angegeben.', mtError, [mbCancel], 0);
      exit;
    end;

    SetControlsStatus(true);

    // Schnittstelle ffnen
    if OpenPort(Sender) <> FT_OK then
      exit;

    // Datei ffnen, und Zeiger auf Anfang der Datei setzen
    AssignFile(MotDatei, DateiNameEdit.Text);
    Reset(MotDatei);
    DateiOpenFlag := true;

    // Erste Zeile lesen und berprfen
    Readln(MotDatei, MotZeile);
    if (LeftStr(MotZeile, 2) <> 'S0') then
    begin
      MessageDlg('Datei ist nicht vom Typ MOT.', mtError, [mbCancel], 0);
      exit;
    end;

    // MC initialisieren, Baudrate setzen und ID prfen
    if not InitChip() then
    begin
      MessageDlg('Der Speicher des MCs ist geschtzt.', mtError, [mbOK], 0);
      exit
    end;

    if TEdit(sender).Name = 'AutoBtn' then
    begin
      Status := BlankCheck();
      if Status < 0 then
        exit
      else
        if Status > 0 then
        begin
          FlashMemo.Lines.Add('Flash-Speicher wird gelschen');
          FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
                format('Chip gelscht. Status: %0.4x     ', [EraseChip()]);
          FlashMemo.Lines.Add('');
        end;

      if FlashFile(FLASHMODUS_WRITE + FLASHMODUS_VERIFY) = 0 then
      begin
        ChangeFTDIData();
        FlashMemo.Lines.Add('Erfolgreich abgeschlossen');
      end;
    end;

    if TEdit(sender).Name = 'FlashBtn' then
      if FlashFile(FLASHMODUS_WRITE) = 0 then
      begin
        ChangeFTDIData();
        FlashMemo.Lines.Add('Erfolgreich abgeschlossen');
      end;

    if TEdit(sender).Name = 'VerifyBtn' then
    begin
      if FlashFile(FLASHMODUS_VERIFY) = 0 then
        FlashMemo.Lines.Strings[FlashMemo.Lines.Count - 1] :=
          'Flash-Speicher stimmt berein';
    end;
  finally
    SetControlsStatus(false);
    if DateiOpenFlag then
      CloseFile(MotDatei);

    // Schnittstelle schlieen
    FT_Close(hDongle);

//    ControlsEnable;
    FlashMemo.Lines.Add('------------------------------------');
  end;
end;

procedure TFirmwareForm.ChangeFTDIData();
var
  DongleDaten: TFtProgramData;
  Hersteller: array[0..40] of char;
  HerstellerID: array[0..40] of char;
  Beschreibung: array[0..40] of char;
  SerienNr: array[0..8] of char;
begin
  DongleDaten.Signature1 := $00000000;
  DongleDaten.Signature2 := $ffffffff;
  DongleDaten.Version := 2;
  DongleDaten.Manufacturer := Hersteller;
  DongleDaten.ManufacturerId := HerstellerID;
  DongleDaten.Description := Beschreibung;
  DongleDaten.SerialNumber := SerienNr;

  if FT_EE_Read(hDongle, @DongleDaten) = FT_OK then
  begin
    if Beschreibung <> DONGLE_BESCHREIBUNG then
    begin
      Beschreibung := DONGLE_BESCHREIBUNG;
      FT_EE_Program(hDongle, @DongleDaten);
      FT_CyclePort(hDongle);
    end;
  end;
end;

procedure TFirmwareForm.SerienNrComboBoxDropDown(Sender: TObject);
var
  i: integer;
  j, n: dword;
  DongleInfo: array of TFtDeviceInfo;
  tempComboText: string;
begin
  i := pos('; ', SerienNrComboBox.Text) - 1;
  if i < 0 then
    i := length(SerienNrComboBox.Text);
  tempComboText := copy(SerienNrComboBox.Text, 0, i);
  SerienNrComboBox.Items.Clear;

  if FT_CreateDeviceInfoList(@n) <> FT_OK then
    exit;

  SetLength(DongleInfo, n);
  if FT_GetDeviceInfoList(@DongleInfo[0], @n) <> FT_OK then
    exit;

  for j := 0 to n-1 do
  begin
    SerienNrComboBox.Items.Add(DongleInfo[j].SerialNumber + '; ' +
          DongleInfo[j].Description);
    if DongleInfo[j].SerialNumber = tempComboText then
      SerienNrComboBox.ItemIndex := j;
  end;
end;

end.
