unit Oszillogramm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, EmpfangPrototyp, ExtCtrls, StdCtrls, Buttons, Hauptfenster,
  IntegerTerm;

type
  TWertrecord = record
    Wert: integer;
    Timestamp: longword;
  end;
  PWertrecord = ^TWertrecord;

  TOszillogrammForm = class(TEmpfangPrototypForm)
    AktivChecker: TCheckBox;
    KonfigBtn: TBitBtn;
    RahmenBevel: TBevel;
    SpeichernBtn: TBitBtn;
    LoeschenBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure KonfigBtnClick(Sender: TObject);
    procedure SpeichernBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    WerteListe: TList;
    BerechnungsObj: TIntTerm;
    Varis: VarArray;

    SchattenBild: TBitmap;  // Speichert das Bild im Hintergrund
    HintergrundBild: TBitmap;
    GrafLeft: integer;
    GrafTop: integer;
    FarbeHinterG: TColor;
    FarbeGitter: TColor;
    FarbeGraf: Tcolor;
    Margin: integer;
    DataPtr: integer;
    MinWert: integer;
    MaxWert: integer;
    NachrichtenID: longword;
    BerechnungsTerm: string;

    Dateiname: string;
    procedure ListeLeeren(Sender: TObject);
    procedure HintergrundErzeugen();
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

implementation

uses AnalogKonfig;

{$R *.dfm}

procedure TOszillogrammForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  inherited;
  TMainForm(owner).ButtonImages.GetBitmap(12, KonfigBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(4, SpeichernBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(1, LoeschenBtn.Glyph);

  Dateiname := '';

  BerechnungsObj := TIntTerm.Create;
  HintergrundBild := TBitmap.Create;
  SchattenBild := TBitmap.Create;

  SetLength(Varis, 8);
  for i := 0 to high(Varis) do
  begin
    Varis[i].Name := format('d%u',[i]);
    Varis[i].Wert := 0;
  end;

  WerteListe := TList.Create;
  WerteListe.Capacity := 1024;

  FarbeHinterG := rgb(10,50,10);
  FarbeGitter := rgb(10,65,10);
  FarbeGraf := rgb(220,200,10);
  Margin := 5;
  SchattenBild.Width := 400;
  SchattenBild.Height := 101;
  RahmenBevel.Left := Margin;
  RahmenBevel.Top := KonfigBtn.Top + KonfigBtn.Height + Margin;
  RahmenBevel.Width := SchattenBild.Width + 2;
  RahmenBevel.Height := SchattenBild.Height + 2;
  GrafLeft := RahmenBevel.Left + 1;
  GrafTop := RahmenBevel.Top + 1;
  self.ClientWidth := SchattenBild.Width + (2 * Margin) + 2;
  self.ClientHeight := GrafTop + SchattenBild.Height + Margin + 2;

  DataPtr := 0;
  MinWert := 0;
  MaxWert := 10000;
  NachrichtenID := 0;
  BerechnungsTerm := '(d0 << 8) + d1';

  HintergrundErzeugen;
end;

procedure TOszillogrammForm.FormDestroy(Sender: TObject);
begin
  inherited;
  ListeLeeren(self);
  WerteListe.Free;
  SchattenBild.Free;
  HintergrundBild.Free;
end;

procedure TOszillogrammForm.FormPaint(Sender: TObject);
var
  i: integer;
  Offset: integer;
begin
  inherited;

  BitBlt(SchattenBild.Canvas.Handle, 0, 0, SchattenBild.Width, SchattenBild.Height,
        HintergrundBild.Canvas.Handle, (WerteListe.Count mod 10), 0, SRCCOPY);

  SchattenBild.Canvas.Pen.Color := FarbeGraf;
  Offset := WerteListe.Count - SchattenBild.Width;
  i := 0;
  while (i < SchattenBild.Width) do
  begin
    if (Offset + i) > 0 then
    begin
      SchattenBild.Canvas.MoveTo(i - 1, (SchattenBild.Height - 1) -
            round((PWertrecord(WerteListe[Offset + i - 1]).Wert - MinWert) *
            (SchattenBild.Height / (MaxWert - MinWert))));
      SchattenBild.Canvas.LineTo(i, (SchattenBild.Height - 1) -
            round((PWertrecord(WerteListe[Offset + i]).Wert - MinWert) *
            (SchattenBild.Height / (MaxWert - MinWert))));
    end;
    inc(i);
  end;

  BitBlt(self.Canvas.Handle, GrafLeft, GrafTop, SchattenBild.Width, SchattenBild.Height,
        SchattenBild.Canvas.Handle, 0, 0, SRCCOPY);
end;

procedure TOszillogrammForm.HintergrundErzeugen;
var
  i: integer;
begin
  HintergrundBild.Width := SchattenBild.Width + 10;
  HintergrundBild.Height := SchattenBild.Height;

  HintergrundBild.Canvas.Pen.Color := FarbeHinterG;
  HintergrundBild.Canvas.Brush.Color := FarbeHinterG;
  HintergrundBild.Canvas.Rectangle(0, 0, HintergrundBild.Width, HintergrundBild.Height);

  HintergrundBild.Canvas.Pen.Color := FarbeGitter;
  i := 0;
  while (i < HintergrundBild.Width) do
  begin
    if (({DataPtr - }HintergrundBild.Width + 1 + i) mod 10 ) = 0 then
    begin
      HintergrundBild.Canvas.MoveTo(i, 0);
      HintergrundBild.Canvas.LineTo(i, HintergrundBild.Height - 1);
    end;
    inc(i);
  end;

  i := 0;
  while (i < HintergrundBild.Height) do
  begin
    HintergrundBild.Canvas.MoveTo(0, i);
    HintergrundBild.Canvas.LineTo(HintergrundBild.Width - 1, i);
    inc(i, 10);
  end;
end;

procedure TOszillogrammForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
  wert: integer;
  PWertRec: PWertrecord;
begin
  if (not AktivChecker.Checked) or (CANrec.ID <> NachrichtenID) then
    exit;

  for i := 0 to high(Varis) do
    Varis[i].Wert := CANrec.Data[i];

  try
    wert := BerechnungsObj.TermLoesen(BerechnungsTerm, @Varis);

    if WerteListe.Count = WerteListe.Capacity then
      WerteListe.Capacity := WerteListe.Capacity + 1024;
    new(PWertRec);
    PWertRec.Wert := wert;
    PWertRec.Timestamp := CANrec.Timestamp;
    WerteListe.Add(PWertRec);

    self.Paint;
  except
    AktivChecker.Checked := false;
    MessageDlg('Im Berechnungs Term ist ein Fehler aufgetretten:'+#13+#10+
          BerechnungsObj.GetFehlerText, mtError, [mbOk], 0);
  end;
end;

procedure TOszillogrammForm.ListeLeeren(Sender: TObject);
begin
  while (WerteListe.Count > 0) do
  begin
    dispose(WerteListe.Items[0]);
    WerteListe.Delete(0);
  end;
  DataPtr := 0;
end;

procedure TOszillogrammForm.KonfigBtnClick(Sender: TObject);
var
  Form: TAnalogKonfigForm;
begin
  Form := TAnalogKonfigForm.Create(self);

  Form.NameEdit.Text := self.Caption;
  Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
//  Form.EinheitEdit.Text := Einheit;
  Form.BerechnungsTermEdit.Text := BerechnungsTerm;
  Form.MinSkalaSpin.Value := MinWert;
  Form.MaxSkalaSpin.Value := MaxWert;
//  Form.SkalaUnterteilungSpin.Value := SkalenSubTeilung;
//  Form.SkalaTeilungSpin.Value := SkalenTeilung;
//  Form.FaktorSpin.Value := SkalenFaktor;

  Form.EinheitEdit.Enabled := false;
  Form.SkalaUnterteilungSpin.Enabled := false;
  Form.SkalaTeilungSpin.Enabled := false;
  Form.FaktorSpin.Enabled := false;

  if Form.ShowModal = mrOK then
  begin
    self.Caption := Form.NameEdit.Text;
    NachrichtenID := Form.CANIDEdit.Tag;
    BerechnungsTerm := Form.BerechnungsTermEdit.Text;
    MinWert := Form.MinSkalaSpin.Value;
    MaxWert := Form.MaxSkalaSpin.Value;

    HintergrundErzeugen;
    self.Paint;
    FensterMenuItem.Caption :=  self.Caption;
  end;
  Form.Free;
end;

procedure TOszillogrammForm.SpeichernBtnClick(Sender: TObject);
var
  i: integer;
  Datei: TextFile;
begin
  TMainForm(owner).SaveDialog.Title := 'Daten-Log speichern';
  TMainForm(owner).SaveDialog.DefaultExt := 'csv';
  if Dateiname = '' then
    TMainForm(Owner).SaveDialog.FileName := self.Caption + '.csv'
  else
    TMainForm(Owner).SaveDialog.FileName := Dateiname;
  TMainForm(owner).SaveDialog.Filter := 'Daten-Log (*.csv)|*.csv|Alle Dateien (*.*)|*.*';

  if TMainForm(owner).SaveDialog.Execute then
  begin
    Dateiname := TMainForm(owner).SaveDialog.FileName;
    try
      AssignFile(Datei, TMainForm(owner).SaveDialog.FileName);
      Rewrite(Datei);
      Writeln(Datei, 'Zeitstempel;Datenwert');
      for i:=0 to WerteListe.Count-1 do
        Writeln(Datei, format('%u;%u', [PWertrecord(WerteListe[i]).Timestamp, PWertrecord(WerteListe[i]).Wert]));
    finally
      CloseFile(Datei);
    end;
  end;
end;

procedure TOszillogrammForm.KonfigLaden(WerteListe: TStrings);
begin
  self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
  NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
  BerechnungsTerm := ReadListString(WerteListe, 'Term', BerechnungsTerm);
  MinWert := ReadListInteger(WerteListe, 'MinWert', MinWert);
  MaxWert := ReadListInteger(WerteListe, 'MaxWert', MaxWert);

  HintergrundErzeugen;
  self.Paint;
  FensterMenuItem.Caption :=  self.Caption;
end;

procedure TOszillogrammForm.KonfigSichern(WerteListe: TStrings);
begin
  WerteListe.Append(format('Name=%s', [self.Caption]));
  WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
  WerteListe.Append(format('Term=%s', [BerechnungsTerm]));
  WerteListe.Append(format('MinWert=%d', [MinWert]));
  WerteListe.Append(format('MaxWert=%d', [MaxWert]));
end;

end.
