unit Senden;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ExtCtrls, StdCtrls, Buttons, StrUtils, Menus, mmsystem;

const
  WM_SENDETIMER = WM_USER + 10;

type
  TCANTransmitRecord = record
    ID: longword;
    DLC: byte;
    Data: array[0..7] of byte;
    Slot: byte;
    Flags: byte; // Bit0/1 = Rx/Tx Remote, Bit2 = Autosend
    Interval: longword;
    Countdown: longword;
  end;
  PCANTransmitRecord = ^TCANTransmitRecord;

  TTimerThread = class;

  TSendenForm = class(TForm)
    NachrichtenGrid: TStringGrid;
    KopfPanel: TPanel;
    HinzufuegenBtn: TBitBtn;
    EntfernenBtn: TBitBtn;
    ListeLeerenBtn: TBitBtn;
    LadenBtn: TBitBtn;
    SpeichernBtn: TBitBtn;
    GeneralChecker: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListeLeeren(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HinzufuegenBtnClick(Sender: TObject);
    procedure NachrichtenGridDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure NachrichtenGridMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure NachrichtenGridMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure NachrichtenGridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure NachrichtenGridGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: String);
    procedure NachrichtenGridSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: String);
    procedure NachrichtenGridKeyPress(Sender: TObject; var Key: Char);
    procedure EntfernenBtnClick(Sender: TObject);
    procedure SpeichernBtnClick(Sender: TObject);
    procedure LadenBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    PrecisionTimer: TTimerThread;

    FensterMenuItem: TMenuItem;
    NachrichtenListe: TList;
    PressedBtn: integer;
    procedure NachrichtSenden(Nr: integer);
    procedure OnPrecisionTimer(var Msg:TMessage);message WM_SENDETIMER;
  public
    { Public-Deklarationen }
    procedure BerechneColWidth();
  end;

  TTimerThread = class(TThread)
  private
    hOwner: HWND;
  protected
    procedure Execute; override;
  public
    constructor Create(AOwner: TForm); //override;
  end;

const
  SlotStatImage: array [0..3] of integer = (0,3,2,4);

implementation

uses Hauptfenster, Protokoll;

{$R *.dfm}

{ TSendenForm }

procedure TSendenForm.FormCreate(Sender: TObject);
begin
  FensterMenuItem := TMainForm(owner).MenuMDIClientHinzufuegen(self);

  TMainForm(owner).ButtonImages.GetBitmap(2, HinzufuegenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(3, EntfernenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(1, ListeLeerenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(5, LadenBtn.Glyph);
  TMainForm(owner).ButtonImages.GetBitmap(4, SpeichernBtn.Glyph);

  PressedBtn := 0;
  NachrichtenListe := TList.Create;
  HinzufuegenBtnClick(Self);
  TimeSetEvent(0, 0, @PressedBtn, self.Handle, TIME_PERIODIC);
  timegettime;

  NachrichtenGrid.ColWidths[0] := 80;
  NachrichtenGrid.ColWidths[1] := 27;
  BerechneColWidth;
  NachrichtenGrid.ColWidths[10] := 27;
  NachrichtenGrid.ColWidths[11] := 30;
  NachrichtenGrid.ColWidths[12] := 30;
  NachrichtenGrid.ColWidths[13] := 56;
  NachrichtenGrid.ColWidths[14] := 56;
  NachrichtenGrid.Hint := 'ID - Idenifier'+#13+
                          'DLC - Lnge der CAN-Nachricht'+#13+
                          'Dx - Daten der Nachricht'+#13+
                          'Slot - Slot Nr. die zum senden der Nachricht genutzt wird'+#13+
                          'Rem - Remote'+#13+
                          'Auto - Schaltet das automatische Senden der Nachricht ein bzw. aus'+#13+
                          'Intrv - Interval, indem die Nachricht wiederholt gesendet wird'+#13+
                          'Senden - Nachricht sofort senden';

  PrecisionTimer := TTimerThread.Create(self);
end;

procedure TSendenForm.FormDestroy(Sender: TObject);
begin
  PrecisionTimer.Terminate;
  PrecisionTimer.Free;
  ListeLeeren(self);
  NachrichtenListe.Free;
end;

procedure TSendenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  TMainForm(owner).MenuMDIClientEntfernen(FensterMenuItem);
  Action := caFree;
end;

procedure TSendenForm.ListeLeeren(Sender: TObject);
begin
  while (NachrichtenListe.Count > 0) do
  begin
    dispose(NachrichtenListe.Items[0]);
    NachrichtenListe.Delete(0);
  end;
  NachrichtenGrid.RowCount := 2;
  NachrichtenGrid.Refresh;
end;

procedure TSendenForm.HinzufuegenBtnClick(Sender: TObject);
var
  PNachricht: PCANTransmitRecord;
  i: integer;
begin
  new(PNachricht);
  PNachricht.ID := 0;
  PNachricht.DLC := 0;
  for i := 0 to high(PNachricht.Data) do
    PNachricht.Data[i] := 0;
  PNachricht.Slot := 0;
  PNachricht.Flags := 0;
  PNachricht.Interval := 0;
  PNachricht.Countdown := PNachricht.Interval;
  NachrichtenListe.Add(PNachricht);
  NachrichtenGrid.RowCount := NachrichtenListe.Count+1;
  NachrichtenGrid.Refresh;
end;

procedure TSendenForm.NachrichtenGridDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  Text: string;
  x,y: integer;
begin
  NachrichtenGrid.Canvas.Brush.Color := NachrichtenGrid.Color;
  NachrichtenGrid.Canvas.Font.Color := clWindowText;

  if gdFocused in State then
  begin
    NachrichtenGrid.Canvas.Brush.Color := clActiveCaption;
    NachrichtenGrid.Canvas.Font.Color := clCaptionText;
  end
  else
    if gdFixed in State then
      NachrichtenGrid.Canvas.Brush.Color := NachrichtenGrid.FixedColor;
  NachrichtenGrid.Canvas.FillRect(Rect);

  if ARow = 0 then
  begin
    case ACol of
      0:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'ID');
      1:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'DLC');
      2:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D0');
      3:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D1');
      4:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D2');
      5:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D3');
      6:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D4');
      7:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D5');
      8:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D6');
      9:  NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'D7');
      10: NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'Slot');
      11: NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'Rem');
      12: NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'Auto');
      13: NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'Interval');
      14: NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,'Senden');
    end;
  end
  else
  begin
    if (NachrichtenListe.Count = 0) then
      exit;

    case ACol of
      0:  Text := TMainForm(Owner).GetIDString(PCANTransmitRecord(NachrichtenListe[ARow-1]).ID);

      1:  Text := format('%u',[PCANTransmitRecord(NachrichtenListe[ARow-1]).DLC]);

      2..9: Text := TMainForm(Owner).GetDataString(PCANTransmitRecord(NachrichtenListe[ARow-1]).Data[ACol-2]);

      10: Text := format('%u',[PCANTransmitRecord(NachrichtenListe[ARow-1]).Slot]);

      11: begin     //  Remote
            x := (NachrichtenGrid.ColWidths[ACol] - TMainForm(Owner).CheckerImages.Width) div 2;
            y := (NachrichtenGrid.DefaultRowHeight - TMainForm(Owner).CheckerImages.Height) div 2;
            TMainForm(self.Owner).CheckerImages.Draw(NachrichtenGrid.Canvas,Rect.Left+x,Rect.Top+y,SlotStatImage[PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags and $03]);
          end;

      12: begin     // Aktiv
            x := (NachrichtenGrid.ColWidths[ACol] - TMainForm(Owner).CheckerImages.Width) div 2;
            y := (NachrichtenGrid.DefaultRowHeight - TMainForm(Owner).CheckerImages.Height) div 2;
            TMainForm(self.Owner).CheckerImages.Draw(NachrichtenGrid.Canvas,Rect.Left+x,Rect.Top+y,(PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags shr 2) and $01);
          end;

      13: Text := format('%u',[PCANTransmitRecord(NachrichtenListe[ARow-1]).Interval]);

      14: begin     // Senden
            with NachrichtenGrid.Canvas do
            begin
              Brush.Color := clBtnFace;
              FillRect(Rect);

              if (PressedBtn <> ARow) then
                Pen.Color := clBtnHighlight
              else
                Pen.Color := clBtnShadow;
              MoveTo(Rect.Left, Rect.Bottom-1);
              LineTo(Rect.Left, Rect.Top);
              LineTo(Rect.Right-1, Rect.Top);
              if (PressedBtn <> ARow) then
                Pen.Color := clBtnShadow
              else
                Pen.Color := clBtnHighlight;
              LineTo(Rect.Right-1, Rect.Bottom-1);
              LineTo(Rect.Left, Rect.Bottom-1);

              Text := 'Senden';
              Font.Color := clBtnText;
              x := (NachrichtenGrid.ColWidths[ACol] - TextWidth(Text)) div 2;
              y := (NachrichtenGrid.DefaultRowHeight - TextHeight(Text)) div 2;
              if (PressedBtn = ARow) then
              begin
                inc(x);
                inc(y);
              end;
              TextOut(Rect.Left+x,Rect.Top+y,Text);
            end;
          end;
    end;
    if not (ACol in [11,12,14]) then
      NachrichtenGrid.Canvas.TextOut(Rect.Left+1,Rect.Top+2,Text);

  end;

  if (ACol = NachrichtenGrid.Col) and (ARow = NachrichtenGrid.Row) and (ACol <> 14) then
    NachrichtenGrid.Canvas.DrawFocusRect(Rect);

end;

procedure TSendenForm.NachrichtenGridMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ACol, ARow: integer;
begin
  NachrichtenGrid.MouseToCell(X, Y, ACol, ARow);
  if (ACol = 14) then       // Senden Schaltflche
    PressedBtn := ARow;
end;

procedure TSendenForm.NachrichtenGridMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ACol, ARow: integer;
  Key: char;
begin
  NachrichtenGrid.MouseToCell(X, Y, ACol, ARow);

  if (Button <> mbLeft) then
    Exit;

  PressedBtn := 0;
  if (NachrichtenGrid.Col = 14) then    // Senden Schaltflche
  begin
    NachrichtenGridDrawCell(Sender, NachrichtenGrid.Col, NachrichtenGrid.Row, NachrichtenGrid.CellRect(NachrichtenGrid.Col, NachrichtenGrid.Row), []);
    if (ACol = NachrichtenGrid.Col) and (ARow = NachrichtenGrid.Row) then
      if (ARow <= NachrichtenListe.Count) and (ARow > 0) then
        NachrichtSenden(ARow-1);
    exit;
  end;

  if (ACol = NachrichtenGrid.Col) and (ARow = NachrichtenGrid.Row) then
    if (ACol in [11,12]) then   // Remote und Auto Spalte
    begin
      Key := ' ';
      NachrichtenGridKeyPress(Sender, Key);
    end;
end;

procedure TSendenForm.NachrichtenGridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  if (ACol in [11,12,14]) then
    NachrichtenGrid.Options := NachrichtenGrid.Options - [GoEditing]
  else
    NachrichtenGrid.Options := NachrichtenGrid.Options + [GoEditing];
end;

procedure TSendenForm.NachrichtenGridGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: String);
begin
  if (NachrichtenGrid.Row > NachrichtenListe.Count) then
    exit;

  case ACol of
    0:  Value := TMainForm(Owner).GetIDString(PCANTransmitRecord(NachrichtenListe[ARow-1]).ID);
    1:  Value := IntToStr(PCANTransmitRecord(NachrichtenListe[ARow-1]).DLC);
    2..9: Value := TMainForm(Owner).GetDataString(PCANTransmitRecord(NachrichtenListe[ARow-1]).Data[ACol-2]);
    10: Value := IntToStr(PCANTransmitRecord(NachrichtenListe[ARow-1]).Slot);
    13: Value := IntToStr(PCANTransmitRecord(NachrichtenListe[ARow-1]).Interval);
  end;
end;

procedure TSendenForm.NachrichtenGridSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: String);
begin
  if (NachrichtenGrid.Row > NachrichtenListe.Count) then
    exit;

  case ACol of
    0:  TMainForm(Owner).CheckZahlenEingabe(Value,
          PCANTransmitrecord(NachrichtenListe[ARow-1]).ID, TMainForm(Owner).MaxID);
    1:  TMainForm(Owner).CheckZahlenEingabe(Value,
          PCANTransmitrecord(NachrichtenListe[ARow-1]).DLC, 8);
    2..9: TMainForm(Owner).CheckZahlenEingabe(Value,
            PCANTransmitrecord(NachrichtenListe[ARow-1]).Data[ACol-2], 255);
    10: TMainForm(Owner).CheckZahlenEingabe(Value,
          PCANTransmitrecord(NachrichtenListe[ARow-1]).Slot, 13);
    13:
      begin
        TMainForm(Owner).CheckZahlenEingabe(Value,
              PCANTransmitrecord(NachrichtenListe[ARow-1]).Interval, 60000);
        PCANTransmitRecord(NachrichtenListe[ARow-1]).CountDown :=
              PCANTransmitrecord(NachrichtenListe[ARow-1]).Interval;
      end;
  end;
end;

procedure TSendenForm.NachrichtenGridKeyPress(Sender: TObject; var Key: Char);
var
  ACol,ARow: integer;
begin
  if (NachrichtenGrid.Row > NachrichtenListe.Count) then
  begin
    Key := #0;
    exit;
  end;

  if Key = ' ' then
  begin
    ACol := NachrichtenGrid.Col;
    ARow := NachrichtenGrid.Row;
    case NachrichtenGrid.Col of
      11: begin
            PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags :=
                (PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags and $fc) + ((PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags + 1) and $03);
            if (PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags and $03) = $01 then
              PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags := PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags + 1;
          end;
      12: PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags := PCANTransmitRecord(NachrichtenListe[ARow-1]).Flags xor $04;
    end;
    NachrichtenGridDrawCell(self, ACol, ARow, NachrichtenGrid.CellRect(ACol, ARow), []);
  end;
end;

procedure TSendenForm.EntfernenBtnClick(Sender: TObject);
begin
  dispose(NachrichtenListe.Items[NachrichtenGrid.Row-1]);
  NachrichtenListe.Delete(NachrichtenGrid.Row-1);
  if NachrichtenListe.Count > 0 then
    NachrichtenGrid.RowCount := NachrichtenListe.Count+1
  else
    HinzufuegenBtnClick(Self);
  NachrichtenGrid.Refresh;
end;

procedure TSendenForm.BerechneColWidth;
var
  i,w: integer;
begin
  if not TMainForm(Owner).actBinDaten.Checked then
    w := NachrichtenGrid.Canvas.TextWidth('$@@')
  else
    w := NachrichtenGrid.Canvas.TextWidth('000000000');

  if (NachrichtenGrid.ColWidths[2] = w) then
    NachrichtenGrid.Refresh
  else
    for i:=2 to 9 do
      NachrichtenGrid.ColWidths[i] := w;
end;

procedure TSendenForm.NachrichtSenden(Nr: integer);
var
  i: integer;
  pLong: plongword;
  Buffer: array[0..13] of Byte;
begin
  PSlotInfo(TMainForm(owner).SlotList[PCANTransmitRecord(NachrichtenListe[Nr]).Slot]).ID :=
        PCANTransmitRecord(NachrichtenListe[Nr]).ID;

  if (PCANTransmitRecord(NachrichtenListe[Nr]).Flags and $01) = 0 then
  begin
    // Slot fr Senden vorbereiten, Normal und auch remote
    Buffer[1] := CANMSGSEND;
    PSlotInfo(TMainForm(owner).SlotList[PCANTransmitRecord(NachrichtenListe[Nr]).Slot]).Flags :=
        $02 + ((PCANTransmitRecord(NachrichtenListe[Nr]).Flags shl 1) and $04);
    if (PCANTransmitRecord(NachrichtenListe[Nr]).Flags and $03) <> 0 then
    begin
      // Remote-Anforderung senden
      Buffer[1] := Buffer[1] or $20;
    end;
  end
  else
  begin
    // Slot fr Remote-Kommunikation vorbereiten
    Buffer[1] := CANMSGEMPGANG or $20;
    if PCANTransmitRecord(NachrichtenListe[Nr]).DLC = 0 then
    begin
      PCANTransmitRecord(NachrichtenListe[Nr]).Flags := PCANTransmitRecord(NachrichtenListe[Nr]).Flags and $fd;
    end
    else
    begin
      // Damit alle Datenbytes bei Remote-auto-response initialisiert sind, alle 8 Datenbytes bertragen
      PCANTransmitRecord(NachrichtenListe[Nr]).DLC := 8;
    end;
    PSlotInfo(TMainForm(owner).SlotList[PCANTransmitRecord(NachrichtenListe[Nr]).Slot]).Flags :=
        (PCANTransmitRecord(NachrichtenListe[Nr]).Flags and $03) + $04;
  end;

  if TMainForm(owner).actStandardID.Checked then
  Begin
    Buffer[0] := 4;
  end
  else
  begin
    Buffer[0] := 6;
    Buffer[1] := Buffer[1] or $10;
  end;

  Buffer[1] := Buffer[1] or (PCANTransmitRecord(NachrichtenListe[Nr]).Slot and $0f);

  pLong := @Buffer[2];
  pLong^ := PCANTransmitRecord(NachrichtenListe[Nr]).ID;

  for i:= 0 to PCANTransmitRecord(NachrichtenListe[Nr]).DLC-1 do
  begin
    Buffer[Buffer[0]] := PCANTransmitRecord(NachrichtenListe[Nr]).Data[i];
    inc(Buffer[0]);
  end;

  TMainForm(owner).SendCommand(@Buffer, Buffer[0]);

  TMainForm(owner).SlotGrid.Refresh;

  if assigned(TMainForm(owner).ProtokollForm) then
    TProtokollForm(TMainForm(owner).ProtokollForm).Memo.Lines.Add(format('Senden Nr.: %u',[Nr]));

  TMainForm(owner).IncTxZaehler;
end;

procedure TSendenForm.SpeichernBtnClick(Sender: TObject);
var
  i: integer;
  s: string;
  Datei: TextFile;
begin
  TMainForm(owner).SaveDialog.Title := 'CAN-Sendeliste speichern';
  TMainForm(owner).SaveDialog.DefaultExt := 'csv';
  TMainForm(owner).SaveDialog.FileName := TMainForm(owner).SendenDatei;
  TMainForm(owner).SaveDialog.Filter := 'CAN-Log (*.csv)|*.csv|Alle Dateien (*.*)|*.*';

  if TMainForm(owner).SaveDialog.Execute then
  begin
    TMainForm(owner).SendenDatei := TMainForm(owner).SaveDialog.FileName;
    try
      AssignFile(Datei, TMainForm(owner).SaveDialog.FileName);
      Rewrite(Datei);
      Writeln(Datei, 'ID;DLC;D0;D1;D2;D3;D4;D5;D6;D7;Rem.;Zeit;Interval;Slot;Flag');
      for i:=0 to NachrichtenListe.Count-1 do
      begin
        s := format('%u;%u;%u;%u;%u;%u;%u;%u;%u;%u;0;;%u;%u;%u',
              [PCANTransmitRecord(NachrichtenListe[i]).ID,
               PCANTransmitRecord(NachrichtenListe[i]).DLC,
               PCANTransmitRecord(NachrichtenListe[i]).Data[0],
               PCANTransmitRecord(NachrichtenListe[i]).Data[1],
               PCANTransmitRecord(NachrichtenListe[i]).Data[2],
               PCANTransmitRecord(NachrichtenListe[i]).Data[3],
               PCANTransmitRecord(NachrichtenListe[i]).Data[4],
               PCANTransmitRecord(NachrichtenListe[i]).Data[5],
               PCANTransmitRecord(NachrichtenListe[i]).Data[6],
               PCANTransmitRecord(NachrichtenListe[i]).Data[7],
               PCANTransmitRecord(NachrichtenListe[i]).Interval,
               PCANTransmitRecord(NachrichtenListe[i]).Slot,
               PCANTransmitRecord(NachrichtenListe[i]).Flags and $fb
              ]);
        Writeln(Datei, s);
      end;
    finally
      CloseFile(Datei);
    end;
  end;
end;

procedure TSendenForm.LadenBtnClick(Sender: TObject);
var
  i,j,n: integer;
  s: string;
  sValue: string;
  Datei: TextFile;
  PNachricht: PCANTransmitRecord;
begin
  TMainForm(owner).OpenDialog.Title := 'CAN-Sendeliste laden';
  TMainForm(owner).OpenDialog.DefaultExt := 'csv';
  TMainForm(owner).OpenDialog.FileName := TMainForm(owner).SendenDatei;
  TMainForm(owner).OpenDialog.Filter := 'CAN-Log (*.csv)|*.csv|Alle Dateien (*.*)|*.*';

  if TMainForm(owner).OpenDialog.Execute then
  begin
    ListeLeeren(self);
    TMainForm(owner).SendenDatei := TMainForm(owner).OpenDialog.FileName;
    try
      AssignFile(Datei, TMainForm(owner).OpenDialog.FileName);
      Reset(Datei);
      Readln(Datei,s);  // ID;DLC;D0;D1;D2;D3;D4;D5;D6;D7;Rem.;Zeit;Interval;Slot;Flag
      while not(eof(Datei)) do // Zeilenschleife
      begin
        new(PNachricht);
        PNachricht.ID := 0;
        PNachricht.DLC := 0;
        for n := 0 to 7 do
          PNachricht.Data[n] := 0;
        PNachricht.Interval := 0;
        PNachricht.Countdown := 0;
        PNachricht.Slot := 0;
        PNachricht.Flags := 0;

        Readln(Datei,s);
        i := 1;

        for n := 0 to 14 do // Spaltenschleife
        begin
          j := PosEx(';', s, i);

          if (j = 0) then
          begin
            if (i > length(s)) then
              break
            else
              j := length(s) + 1;
          end;

          if (j-i > 0) then
            sValue := copy(s, i, j-i)
          else
            sValue := '0';

          case (n) of
            0:  PNachricht.ID := StrToInt(sValue);
            1:  PNachricht.DLC := StrToInt(sValue);
            2..9:
                PNachricht.Data[n-2] := StrToInt(sValue);
            12:
              begin
                PNachricht.Interval := StrToInt(sValue);
                PNachricht.Countdown := PNachricht.Interval;
              end;
            13: PNachricht.Slot := StrToInt(sValue);
            14: PNachricht.Flags := StrToInt(sValue);
          end;
          i := j + 1;
        end;
        NachrichtenListe.Add(PNachricht);
        NachrichtenGrid.RowCount := NachrichtenListe.Count+1;
      end;
    finally
      CloseFile(Datei);
    end;
  end;
  NachrichtenGrid.Refresh;
end;

procedure TSendenForm.FormShow(Sender: TObject);
begin
  self.BringToFront;
  self.WindowState := wsNormal;
end;

procedure TSendenForm.OnPrecisionTimer(var Msg: TMessage);
var
  i: integer;
begin
  if not GeneralChecker.Checked then
    exit;

  for i:=0 to NachrichtenListe.Count-1 do
  begin
    if (PCANTransmitRecord(NachrichtenListe[i]).Flags and $04) <> 0 then
    begin
      if PCANTransmitRecord(NachrichtenListe[i]).Countdown = 0 then
      begin
        NachrichtSenden(i);
        PCANTransmitRecord(NachrichtenListe[i]).Countdown := PCANTransmitRecord(NachrichtenListe[i]).Interval;
      end;
      if PCANTransmitRecord(NachrichtenListe[i]).Countdown > 0 then
        dec(PCANTransmitRecord(NachrichtenListe[i]).Countdown);
    end;
  end;
end;

{ TTimerThread }

constructor TTimerThread.Create(AOwner: TForm);
begin
  hOwner := AOwner.Handle;
  inherited Create(false);
end;

procedure TTimerThread.Execute;
var
  ThreadNameInfo: TThreadNameInfo;
begin
  inherited;

  ThreadNameInfo.FType := $1000;
  ThreadNameInfo.FName := 'TimerThread';
  ThreadNameInfo.FThreadID := $FFFFFFFF;
  ThreadNameInfo.FFlags := 0;
  try
    RaiseException($406D1388, 0, sizeof(ThreadNameInfo) div sizeof(LongWord), @ThreadNameInfo);
  except
  end;

  while not Terminated do
  begin
    sleep(1);
    PostMessage(hOwner, WM_SENDETIMER, 0, 0);
  end;

end;

end.
