unit WertAnzeige;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, EmpfangPrototyp, StdCtrls, Buttons, Hauptfenster, IntegerTerm,
  ExtCtrls;

type
  TWertAnzeigeForm = class(TEmpfangPrototypForm)
    AktivChecker: TCheckBox;
    KonfigBtn: TBitBtn;
    WertLabel: TLabel;
    EinheitLabel: TLabel;
    RahmenBevel: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure KonfigBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    BerechnungsObj: TIntTerm;
    Varis: VarArray;
    NachrichtenID: longword;
    BerechnungsTerm: string;
    Einheit: string;
    Margin: integer;       // Abstand der Skala zum Rand
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

var
  WertAnzeigeForm: TWertAnzeigeForm;

implementation

uses AnalogKonfig;

{$R *.dfm}

procedure TWertAnzeigeForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  inherited;
  TMainForm(owner).CheckerImages.GetBitmap(5, KonfigBtn.Glyph);

  BerechnungsObj := TIntTerm.Create;

  SetLength(Varis, 8);
  for i := 0 to high(Varis) do
  begin
    Varis[i].Name := format('d%u',[i]);
    Varis[i].Wert := 0;
  end;

  Margin := 5;
  Einheit := 'x';
  NachrichtenID := 0;
  BerechnungsTerm := '(d0 << 8) + d1';

  WertLabel.Caption := '0';

  FormResize(self);
end;

procedure TWertAnzeigeForm.FormResize(Sender: TObject);
begin
  inherited;
  RahmenBevel.Left := Margin;
  WertLabel.Left := RahmenBevel.Left + Margin;
  WertLabel.Width := WertLabel.Canvas.TextWidth('888888');
  EinheitLabel.Left := WertLabel.Left + WertLabel.Width + Margin;
  EinheitLabel.Width := EinheitLabel.Canvas.TextWidth(Einheit);
  RahmenBevel.Width := WertLabel.Width + EinheitLabel.Width + (3 * Margin);
  self.ClientWidth := RahmenBevel.Width + (2 * Margin);
  self.ClientHeight := RahmenBevel.Top + RahmenBevel.Height + Margin;
  KonfigBtn.Left := self.ClientWidth - KonfigBtn.Width;
  EinheitLabel.Caption := Einheit;
end;

procedure TWertAnzeigeForm.KonfigLaden(WerteListe: TStrings);
begin
  self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
  NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
  BerechnungsTerm := ReadListString(WerteListe, 'Term', BerechnungsTerm);
  Einheit := ReadListString(WerteListe, 'Einheit', Einheit);

  FormResize(self);
  FensterMenuItem.Caption :=  self.Caption;
end;

procedure TWertAnzeigeForm.KonfigSichern(WerteListe: TStrings);
begin
  WerteListe.Append(format('Name=%s', [self.Caption]));
  WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
  WerteListe.Append(format('Term=%s', [BerechnungsTerm]));
  WerteListe.Append(format('Einheit=%s', [Einheit]));
end;

procedure TWertAnzeigeForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
begin
  if (not AktivChecker.Checked) or (CANrec.ID <> NachrichtenID) then
    exit;

  for i := 0 to high(Varis) do
    Varis[i].Wert := CANrec.Data[i];

  try
    WertLabel.Caption := IntToStr(BerechnungsObj.TermLoesen(BerechnungsTerm, @Varis));
  except
    AktivChecker.Checked := false;
    MessageDlg('Im Berechnungs Term ist ein Fehler aufgetretten:'+#13+#10+
          BerechnungsObj.GetFehlerText, mtError, [mbOk], 0);
  end;
end;

procedure TWertAnzeigeForm.KonfigBtnClick(Sender: TObject);
var
  Form: TAnalogKonfigForm;
begin
  Form := TAnalogKonfigForm.Create(self);

  Form.NameEdit.Text := self.Caption;
  Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
  Form.BerechnungsTermEdit.Text := BerechnungsTerm;
  Form.EinheitEdit.Text := Einheit;
  Form.SkalaGroup.Visible := false;

  if Form.ShowModal = mrOK then
  begin
    self.Caption := Form.NameEdit.Text;
    NachrichtenID := Form.CANIDEdit.Tag;
    BerechnungsTerm := Form.BerechnungsTermEdit.Text;
    Einheit := Form.EinheitEdit.Text;

    FormResize(self);
    FensterMenuItem.Caption :=  self.Caption;
  end;
  Form.Free;
end;

end.
